package helix.graph.algo;

/**
 * compute graph delta closure
 * 
 * AV: je n'ai conserve que le BFS (plus rapide sur tous les tests)
 *     un peu optimise.
 */
import helix.graph.model.Edge;
import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;


public class DeltaClosure {

	protected Graph graph_;
	protected int delta_;
	protected Constraint constraint_;

	//--------
	// BFS Version
	//--------
	
	/**
	 * internal utility : BFS search of neighbors at distance <= delta
	 */
	protected void deltaNeighboursBFS_(GSet<Vertex> kernel, GSet<Vertex> crown, GSet<Vertex> result, GSet<Edge> ignoreEdges, int delta) {
		
		if (delta < 0)
			return;

		result.addAll(kernel);
		crown.clear();
		
		for (Vertex v : kernel) {
			for (Vertex u : v.neighbors()) {
				if (result.contains(u))
					continue;
				Edge e = v.getEdgeTo(u);
				if (ignoreEdges.contains(e))
					continue;
				if (constraint_.accept(e)) {
					crown.add(u);
				}
			}
		}
		
		if (crown.isEmpty())
			return;

		// recurse with kernel = crown
		
		deltaNeighboursBFS_(crown, kernel, result, ignoreEdges, delta-1);
	}

	/**
	 * run delta closure computation
	 * -- BFS version --
	 */
	protected void runBFS_() {

		GSet<Vertex> neighbours = graph_.vertexSetFactory().make();
		GSet<Vertex> kernel = graph_.vertexSetFactory().make();
		GSet<Vertex> crown = graph_.vertexSetFactory().make();

		GSet<Edge> newEdges = graph_.edgeSetFactory().make();
		
		for (Vertex v : graph_.vertices()) {

			neighbours.clear();
			kernel.clear();
			kernel.add(v);
			
			deltaNeighboursBFS_(kernel, crown, neighbours, newEdges, delta_);
			
			for (Vertex u : neighbours) {
				
				if (u == v)
					continue;
				if (! constraint_.validate(u, v)) // optional validate
					continue;
				Edge e = graph_.getEdge(u, v);
				if (e == null) {
					e = graph_.addEdge(u, v);
					newEdges.add(e);
				}
				// note: it is important to call 'register'
				// even if the edge already exists
				// since this method may modify the edge properties
				constraint_.register(e);   // optional register
			}
		}
	}

	/**
	 * API
	 */

	/**
	 * class to define edge constraint
	 */
	public static abstract class Constraint {
		public abstract boolean accept(Edge e); 			    // mandatory
		public boolean validate(Vertex v1, Vertex v2) {			// optional
			return true;
		}
		public void register(Edge e) {}; 						// optional
	}
	
	/**
	 * constructor
	 */

	public DeltaClosure(Graph graph, int delta, Constraint constraint) {
		graph_ = graph;
		delta_ = delta;
		constraint_ = constraint;
	}
	
	/**
	 * run delta closure computation
	 */
	public void run() {
		runBFS_();
	}
}
