package helix.graph.algo;

import helix.graph.model.GraphFactory;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;

/*
 * random graph Erdos-Renyi model
 */
public class ErdosGenerator extends AbstractRandomGenerator {
	
	protected int nbVertices_;
	protected double connectProba_;
	
	public ErdosGenerator(int nbVertices, double connectProba) {
		if (nbVertices < 0)
			throw new IllegalArgumentException("nbVertices should be >= 0");
		if ((connectProba < 0.) || (connectProba > 1.))
			throw new IllegalArgumentException("connection probality should be in [0., 1.]");
		nbVertices_ = nbVertices;
		connectProba_ = connectProba;
	};

	/*
	 * G(n, p) model
	 * generation: O(N^2)
	 */
	
	public Graph generate(GraphFactory factory) {
		Graph g = factory.make();
    	Vertex[] v = new Vertex[nbVertices_];
    	// add vertices
    	for (int i = 0; i < nbVertices_ ; i++) {
    		v[i] = g.addVertex();
    	}
    	// add edges
    	for (int i = 0; i < nbVertices_ - 1 ; i++) {
        	for (int j = i+1; j < nbVertices_ ; j++) {
        		double p = random_.nextDouble();
        		if (p < connectProba_) {
        		    g.addEdge(v[i], v[j]);
        		}
        	}
    	}
    	return g;
	}

}
