package helix.graph.algo;

import helix.graph.io.DimacsExtendedReader;
import helix.graph.io.GraphReader;
import helix.graph.model.Edge;
import helix.graph.model.Graph;
import helix.graph.model.GraphFactory;
import helix.graph.model.Registry;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;
import helix.graph.model.impl.memory.MemGraphFactory;

import java.io.IOException;
import java.io.StringReader;

import org.junit.Test;
import static org.junit.Assert.*;

public class TestDeltaClosure {
	
	@Test
	public void testOne() throws IOException {
		
		String s = "p test\nn label=1 colors=1\nn label=2 colors=1\nn label=3 colors=1\nn label=4 colors=1\n" +
		           "e 1 2\ne 2 3\ne 3 4";
		StringReader sreader = new StringReader(s);
		GraphFactory factory = MemGraphFactory.instance();
		GraphReader reader = new DimacsExtendedReader(sreader, factory);
		Graph g = reader.read();
		Vertex v1 = g.findVertex(Registry.Label, 1);
		Vertex v2 = g.findVertex(Registry.Label, 2);
		Vertex v3 = g.findVertex(Registry.Label, 3);
		Vertex v4 = g.findVertex(Registry.Label, 4);

		DeltaClosure.Constraint sameColor = new DeltaClosure.Constraint() {
			public boolean accept(Edge e) {
				// accept edge (u,v) if both vertices are of the same color
				return RColorSet.getColors(e.v1()).equals(RColorSet.getColors(e.v2()));
			}
		};

		assertTrue("color 1", RColorSet.hasColor(v1, 0));
		assertTrue("color 2", RColorSet.hasColor(v2, 0));
		assertTrue("color 3", RColorSet.hasColor(v3, 0));
		assertTrue("color 4", RColorSet.hasColor(v4, 0));
		
		DeltaClosure algo = new DeltaClosure(g, 0, sameColor);
		algo.run();
		assertFalse("1 3", g.hasEdge(v1, v3));
		assertFalse("1 4", g.hasEdge(v1, v4));
		assertFalse("2 4", g.hasEdge(v2, v4));

		algo = new DeltaClosure(g, 1, sameColor);
		algo.run();
		assertFalse("1 3", g.hasEdge(v1, v3));
		assertFalse("1 4", g.hasEdge(v1, v4));
		assertFalse("2 4", g.hasEdge(v2, v4));
		
		algo = new DeltaClosure(g, 2, sameColor);
		algo.run();
		assertTrue("1 3", g.hasEdge(v1, v3));
		assertFalse("1 4", g.hasEdge(v1, v4));
		assertTrue("2 4", g.hasEdge(v2, v4));

		algo = new DeltaClosure(g, 2, sameColor);
		algo.run();
		assertTrue("1 4", g.hasEdge(v1, v4));
		
	}

	// only test speed
	@Test
	public void testTwo() throws IOException {
		ErdosGenerator gen = new ErdosGenerator(1000, 0.001);
		GraphFactory factory = MemGraphFactory.instance();
		Graph g = gen.generate(factory);
		
		DeltaClosure.Constraint alwaysTrue = new DeltaClosure.Constraint() {
			public boolean accept(Edge e) {
				return true;
			}
		};
		
		DeltaClosure algo = new DeltaClosure(g, 3, alwaysTrue);
		algo.run();
	}
}