package helix.graph.algo;

import helix.graph.io.DimacsReader;
import helix.graph.io.GraphReader;
import helix.graph.model.Graph;
import helix.graph.model.GraphFactory;
import helix.graph.model.Registry;
import helix.graph.model.Vertex;
import helix.graph.model.impl.memory.MemGraphFactory;

import java.io.IOException;
import java.io.StringReader;

import org.junit.Test;

import static org.junit.Assert.*;

public class TestWarshall {
	@Test
	public void testOne() throws IOException {
		
		String s = "p test\nn 1\nn 2\nn 3\nn 4\ne 1 2\ne 1 3\ne 3 4";
		StringReader sreader = new StringReader(s);
		GraphFactory factory = MemGraphFactory.instance();
		GraphReader reader = new DimacsReader(sreader, factory);
		Graph g = reader.read();
		Vertex v1 = g.findVertex(Registry.Label, 1);
		Vertex v2 = g.findVertex(Registry.Label, 2);
		@SuppressWarnings("unused")
		Vertex v3 = g.findVertex(Registry.Label, 3);
		Vertex v4 = g.findVertex(Registry.Label, 4);
		assertTrue("edge 1 2", g.hasEdge(v1,v2));
		
		
		Warshall algo = new Warshall(g);
		algo.run();

		assertTrue(algo.distance(v1, v2) == algo.distance(v2, v1));
		
		assertEquals("1 1 = 0", 0, algo.distance(v1, v1));
		assertEquals("1 2 = 1", 1, algo.distance(v1, v2));
		assertEquals("1 4 = 2", 2, algo.distance(v1, v4));
	}

	// only test speed
	@Test
	public void testTwo() throws IOException {
		ErdosGenerator gen = new ErdosGenerator(100, 0.1);
		GraphFactory factory = MemGraphFactory.instance();
		Graph g = gen.generate(factory);
		
		Warshall algo = new Warshall(g);
		algo.run();
	}
	
	
}
