package helix.graph.algo;

import java.util.Hashtable;

import helix.graph.model.Graph;
import helix.graph.model.Vertex;

public class Warshall {

	private static final int Infinity = Integer.MAX_VALUE;

	protected Graph graph_;
	
	protected Hashtable<Vertex, Hashtable<Vertex, Integer>> matrix_;

	/*
	 * API
	 */

	/**
	 * constructor
	 */

	public Warshall(Graph graph) {
		graph_ = graph;
		
		// initialisation matrice
		matrix_ = new Hashtable<Vertex, Hashtable<Vertex, Integer>>();
		for (Vertex i : graph_.vertices()) {
			Hashtable<Vertex, Integer> row = new Hashtable<Vertex, Integer>();
			matrix_.put(i, row);
			for (Vertex j : graph_.vertices()) {
				int d;
				if (i == j) {
					d = 0;
				}
				else if (graph_.hasEdge(i, j)) {
					d = 1;
				}
				else {
					d = Infinity /2;
				}
				row.put(j, d);
			}
		}
	}

	/**
	 * main Warshall algorithm
	 * get all distances between all pairs of vertices
	 * TODO connected by given color -> add constraint
	 */
	public void run() {
		
		for (Vertex k : graph_.vertices()) {
			Hashtable<Vertex, Integer> rowk = matrix_.get(k);
			for (Vertex i : graph_.vertices()) {
				Hashtable<Vertex, Integer> rowi = matrix_.get(i);
				for (Vertex j : graph_.vertices()) {
				   Hashtable<Vertex, Integer> rowj = matrix_.get(j);
				   int dij = rowi.get(j);
				   int dki = rowk.get(i);
				   int dkj = rowk.get(j);
				   if (dij > dki + dkj) {  
					 int d = dki + dkj;
					 rowi.put(j,d);
					 rowj.put(i,d);
				   }
				}
			}
		}
	}

	/**
	 * get shortest path between vertex u and v
	 */
			
	public int distance(Vertex u, Vertex v) {
		return matrix_.get(u).get(v);
	}

	

}
