package helix.graph.algo.c3part;

/**
 * Compute common connected components of a data graph
 * provides base methods for OTFQGlobal algorithm 
 */

import helix.graph.algo.CCVisitor;
import helix.graph.algo.c3part.visitor.CCSuperStarByColorVisitor;
import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RVertexList;

import java.util.Set;

public abstract class C3PartAbstractOTFQGlobal extends C3PartAbstractOTFQuorum {

	/*
	 * Core methods
	 * AbstractC3PartOTF overriden methods
	 */
	
	@Override
	/**
	 * override splitGraphColor_ to add full stars in usual CC's
	 */ 
	protected Set<Graph> splitGraphColor_(Graph Clas, int color) {
		
		// run the connected components visitor
		// TODO a resetable visitor 

		int colorIndex = C3PartUtil.getOptimizer(Clas).getColorIndex(color);

		CCVisitor visitor = new CCSuperStarByColorVisitor(Clas, color, colorIndex, starVertex_);
		Iterable<GSet<Vertex>> cc = visitor.traverse().getComponents();
		
		// physically split graph
		//
		return Clas.physicalSplit(cc);
	}

	@Override
	/**
	 * override beforeSplitDFS_ to change the mineltsize condition
	 */ 
	protected boolean beforeSplitGraphDFS_(Graph Clas) {

		// make sure we have vertices
		if (Clas.nbVertices() == 0)
			return false;
		
		GSet<Vertex> terminals = setFactory_.make();

		// check if for each color *without stars*, the number of nodes
		// is at least mineltsize

		RVertexList First = C3PartUtil.getNodes(Clas.vertices().iterator().next());
		
		int nbColors = tupleSize_(Clas);
		
		for (int indx = 0 ; indx < nbColors ; indx++) {
			if (First.getVertex(indx) == starVertex_)
				continue;
			terminals.clear();
			for (Vertex V : options_.compressor.getVertices(Clas)) {
				Vertex v = C3PartUtil.getNodes(V).getVertex(indx);
				terminals.add(v); 
				if (terminals.size() >= options_.mineltsize)
					break;
			}
			if (terminals.size() < options_.mineltsize)
				return false;  // condition not satisfied
		}
		
		return true; // condition satisfied for all colors
	}

	/*
	 * required C3PartAlgo API
	 */
	
	public C3PartAbstractOTFQGlobal(Graph dataGraph) {
		super(dataGraph);
	}
	
}
