package helix.graph.algo.c3part;

import helix.graph.algo.c3part.aggregator.NodeAggregator;
import helix.graph.algo.c3part.optimizer.ColorOptimizer;
import helix.graph.model.Graph;

import java.util.Set;

/**
 * interface for all C3Part algorithms
 */

public interface C3PartAlgo {

	/**
	 * run main algo (with current options)
	 */
	public Set<Graph> run();

	
	/**
	 * set algorithm options
	 * note: this may throw an Exception if options are incompatible with this algo
	 */
	public void setOptions(C3PartOptions options);

	/**
	 * get current algorithm options
	 * return a hard copy of options, so modifying them will not modify
	 * internal algorithm options. to do so you should set options back
	 * with <code>setOptions</code>
	 */
	public C3PartOptions getOptions();
		
	/**
	 * get current aggregator for this algo
	 */
	public NodeAggregator getAggregatorFor();

	/**
	 * get current optimizer for this algo and (possibly) this particular Clas
	 */
	public ColorOptimizer getOptimizerFor(Graph Clas);
	
	/*
	 * reserved API
	 */
	
	/**
	 * get current algorithm options
	 * this method returns the internal algo options. Take care
	 * not modifying them (use <code>getOptions</code> instead)
	 */
	public C3PartOptions getPhysicalOptions();

}
