package helix.graph.algo.c3part;

/**
 * Compute common connected components of a multigraph
 * Breadth-first search version
 * 
 */

import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;

import java.util.Set;

public class C3PartBFS extends C3PartAbstractXFS {

	/**
	 * refine partition on given color.
	 * i.e. split its classes into subclasses
	 */
	protected Set<GSet<Vertex>> refineColor_(Set<GSet<Vertex>> partition, int color) {
		Set<GSet<Vertex>> result = setFactory_.makeSuper();
		for (GSet<Vertex> clas : partition) {
			Set<GSet<Vertex>> split = splitSetColor_(clas, color);
			for (GSet<Vertex> cc : split) {
				if (cc.size() >= options_.minsize)
					result.add(cc);
			}
		}
		return result;
	}

	/**
	 * refine partition on all colors
	 */
	protected Set<GSet<Vertex>> refine_(Set<GSet<Vertex>> partition) {
		for (int color : options_.colors) {
			partition = refineColor_(partition, color);
		}
		return partition;
	}

	/**
	 * algorithm special entry point with multigraph
	 */
	protected Set<GSet<Vertex>> runMulti_(Graph multi, C3PartOptions options) {

		// keep old options

		C3PartOptions oldOptions = this.options_;
		this.options_= options;
		
		// run actual algo
		
		Set<GSet<Vertex>> ccc = setFactory_.makeSuper();
		
		// initialize partition with 1 class of all vertices
		GSet<Vertex> first = setFactory_.make();
		first.addIterable(multi.vertices());
		ccc.add(first);

		// main loop : refine until no change in partition
		int previous = 0;
		while (ccc.size() != previous) {
			previous = ccc.size();
			ccc = refine_(ccc);
		}

		// restore options
		
		this.options_= oldOptions;

		return ccc;
	}
	
	/*
	 * API
	 */
	
	/**
	 * constructor
	 */
	public C3PartBFS(Graph dataGraph) {
		super(dataGraph);
	}
	
}
