package helix.graph.algo.c3part;

/**
 * Compute common connected components of a multigraph
 * Depth-first search version
 * Run on multigraph
 */

import java.util.Set;

import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;

public class C3PartDFS extends C3PartAbstractXFS {

	/**
	 *  recursively split Clas on all colors (DFS approach).
	 *  store common connected components into ccc
	 */
	
	protected void splitClassDFS_(GSet<Vertex> Clas, Set<GSet<Vertex>> ccc) {
		
		for (int color : options_.colors) {
			Set<GSet<Vertex>> split = splitSetColor_(Clas, color);
			if (split.size() != 1) {                    // if Clas splitted
				for (GSet<Vertex> sub : split) {        //
					splitClassDFS_(sub, ccc); 			// recurse to subclasses
				}
				return;
			}
		}
		// Clas remain unsplitted after all colors => it's a CCC 
		if (Clas.size() >= options_.minsize)
			ccc.add(Clas);
	}

	/**
	 * algorithm main entry point
	 */
	protected Set<GSet<Vertex>> runMulti_(Graph multi, C3PartOptions options) {
		
		// keep old options
		C3PartOptions oldOptions = this.options_;
		this.options_= options;

		// run actual algo
		
		// initialize partition with 1 class of all vertices
		GSet<Vertex> first = setFactory_.make();
		first.addIterable(multi.vertices());

		// recursively split initial class
		Set<GSet<Vertex>> ccc = setFactory_.makeSuper();
		splitClassDFS_(first, ccc);

		// restore options
		this.options_= oldOptions;

		return ccc;
	}
	
	/**
	 * API
	 */

	public C3PartDFS(Graph dataGraph) {
		super(dataGraph);
	}
	
}
