package helix.graph.algo.c3part;

/**
 * Compute common connected components of a multigraph
 * Depth-first search version
 * Run on multigraph  Still experimental
 */

import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;
import helix.graph.util.IPermutation;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

public class C3PartDFSQ extends C3PartAbstractXFS {
	
	/**
	 * internal class to keep used and skipped colors with sets
	 */
	protected class ColoredGSet {
		
		protected GSet<Vertex> gset;
		protected RColorSet usedColors;
		protected RColorSet skippedColors;

		public ColoredGSet() {
			this.gset = datagraph_.vertexSetFactory().make();
			this.usedColors = new RColorSet();
			this.skippedColors = new RColorSet();
		}
		
		public ColoredGSet(GSet<Vertex> gset) {
			this();
			this.gset.addAll(gset);
		}

		public ColoredGSet(GSet<Vertex> gset, RColorSet usedColors, RColorSet skippedColors) {
			this.gset = datagraph_.vertexSetFactory().make(gset);
			this.usedColors = new RColorSet(usedColors);
			this.skippedColors = new RColorSet(skippedColors);
		}

		public ColoredGSet(GSet<Vertex> gset, ColoredGSet cset) {
			this(gset, cset.usedColors, cset.skippedColors);
		}
		
		public ColoredGSet(ColoredGSet cset) {
			this(cset.gset, cset);
		}
	}

	/**
	 *  recursively split Clas on all colors (DFS approach).
	 *  keep best result only
	 *  TODO : best for all algos
	 */
	
	protected ColoredGSet Best_;
	protected ArrayList<ColoredGSet> BestArray_;
	
	protected void splitClassDFS_(ColoredGSet Clas) {

		if (Clas.gset.size() < options_.mineltsize)			// --- pruning
			return;
		
		if (Clas.gset.size() < Best_.gset.size())			// --- pruning
			return;
		
		for (int color : options_.colors) {
			
			if (Clas.skippedColors.hasColor(color)) {		// --- already skipped
				continue;
			}
			
			else if (Clas.usedColors.hasColor(color)) {		// --- already splitted
				Set<GSet<Vertex>> split = splitSetColor_(Clas.gset, color);
				if (split.size() != 1) {                    // if Clas splitted again
					for (GSet<Vertex> sub : split) {        //    recurse to subclasses
						ColoredGSet csub = new ColoredGSet(sub, Clas);
						splitClassDFS_(csub);
					}
					return;
				}
			}
			else {											// --- new color
				Clas.usedColors.setColor(color);
				Set<GSet<Vertex>> split = splitSetColor_(Clas.gset, color);
				for (GSet<Vertex> sub : split) {        //    recurse to subclasses
					ColoredGSet csub = new ColoredGSet(sub, Clas);
					splitClassDFS_(csub);
				}
				
				if (Clas.skippedColors.countColors() >= options_.maxstar)
					return;
				
				Clas.usedColors.clearColor(color);
				Clas.skippedColors.setColor(color); 
				splitClassDFS_(Clas);
				return;
			}
		}

		// Clas remain unsplitted after all colors => it's a CCC 
		
		if (Clas.gset.size() > Best_.gset.size()) {
			BestArray_.clear();
			System.err.println("Best so far : " + Clas.gset.size());
		}
		if (Clas.gset.size() >= Best_.gset.size()) {
			System.err.println("Best again : " + Clas.gset.size());
			Best_ = new ColoredGSet(Clas);
			BestArray_.add(Best_);
		}
	}

	/**
	 * algorithm main entry point
	 */
	protected ArrayList<ColoredGSet> runMulti_(Graph multi) {

		// TODO check 
		options_.colors = new IPermutation(0, RColorSet.nbColors(multi) - 1);
		// initialize partition with 1 class of all vertices
		GSet<Vertex> first = setFactory_.make();
		first.addIterable(multi.vertices());
		
		// recursively split initial class
		Best_ = new ColoredGSet();
		BestArray_ = new ArrayList<ColoredGSet>();
		ColoredGSet Clas = new ColoredGSet(first);

		splitClassDFS_(Clas);

		return BestArray_;
	}
	
	/**
	 * dummy entry point : do not use
	 */
	protected Set<GSet<Vertex>> runMulti_(Graph multi, C3PartOptions options) {
		return null;
	}

	@Override
	/**
	 * override run_
	 */
	protected Set<Graph> run_() {
		
		Graph multi = datagraph_;  // input is already a multigraph
		assertMultiNodes(multi);   // make sure we do have nodes

		ArrayList<ColoredGSet> bestArray = this.runMulti_(multi);

		// finaly convert CCC as subgraphs to obey the C3PartAlgo interface
		//
		
		System.err.println("# solutions : " + bestArray.size());

		Set<Graph> result = new HashSet<Graph>();

		for (ColoredGSet cgset : bestArray) {
		  HashSet<GSet<Vertex>> set = new HashSet<GSet<Vertex>>(1);
		  set.add(cgset.gset);
		  Set<Graph> split = multi.physicalSplit(set);
		  Graph singleton = split.iterator().next();
		  RColorSet.getColors(singleton).setColors(cgset.usedColors);
		  result.add(singleton);
		}

		return result;
	}
	
	@Override
	/**
	 * our afterRun_ : don't do anything to multi
	 */
	protected Set<Graph> afterRun_(Set<Graph> partition) {
		return partition;
	}

	@Override
	protected C3PartOptions makeDefaultOptions_() {
		C3PartOptions options = super.makeDefaultOptions_();
		options.ismulti = true;
		options.optimizerName = C3PartOptions.NO_OPTIMIZER;
		options.colors = new IPermutation(0, RColorSet.nbColors(datagraph_) - 1);;
		return options;
	}

	@Override
	protected void logAlgoInfo_() {
		
		options_.logger.info("--------------------------------- Algo parameters");
		options_.logger.info("algo_class:       " + this.getClass().getSimpleName());
		options_.logger.info("mineltsize:       " + options_.mineltsize);
		options_.logger.info("maxstar:          " + options_.maxstar);
	}
	
	/**
	 * API
	 */

	public C3PartDFSQ(Graph dataGraph) {
		super(dataGraph);
	}

	@Override
	public void setOptions(C3PartOptions options) {
		
		if (! options.ismulti) {
			options.logger.warning("warning: option ismulti will be turned on for dfsq algorithm");
			options.ismulti = true;
		}
		if (C3PartOptions.NO_OPTIMIZER.compareToIgnoreCase(options.optimizerName) != 0) {
			options.logger.warning("warning: color optimisation will be turned off for dfsq algorithm");
			options.optimizerName = C3PartOptions.NO_OPTIMIZER;
		}
		super.setOptions(options);
	}
	
}
