package helix.graph.algo.c3part;

/**
 * Convert Datagraph to Multigraph using the OTF
 * expandMultigraph method.
 */

import java.util.HashSet;
import java.util.Set;

import helix.graph.algo.c3part.optimizer.ColorOptimizer;
import helix.graph.algo.c3part.optimizer.IdentityColorOptimizer;
import helix.graph.model.Graph;
import helix.graph.model.impl.RColorSet;


public class C3PartDTM extends C3PartAbstractOTF {

	@Override
	/**
	 * there is no need of a color optimizer for DTM since we compute
	 * all the multigraph
	 */

	public ColorOptimizer getOptimizerFor(Graph Clas) {
		return new IdentityColorOptimizer(RColorSet.nbColors(datagraph_) - 1);
	}

	@Override
	/**
	 * algorithm main entry point
	 */
	protected Set<Graph> run_() {

		// init multigraph with first color
		
		Graph multi = initMultiGraph_();

		// run iterative expansion
		
		ColorOptimizer optimizer = C3PartUtil.getOptimizer(multi);
		while (optimizer.hasNext()) {
			multi = expandMultiGraph_(multi, optimizer.next());
		}
		
		// return a singleton (to obey the C3PartAlgo interface)
		//
		Set<Graph> result = new HashSet<Graph>();
		result.add(multi);
		
		return result;
	}

	/*
	 * C3PartAlgo API
	 */

	public C3PartDTM(Graph dataGraph) {
		super(dataGraph);
	}

}
