package helix.graph.algo.c3part;

import helix.graph.algo.c3part.compressor.MultiGraphCompressor;
import helix.graph.algo.c3part.compressor.MultiIdentCompressor;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;
import helix.graph.model.impl.RVertexList;

/**
 * a class for debugging multigraph
 */
public class MultiGraphDebugger {

	// just to access to compressor methods
	private static final MultiGraphCompressor compress_ = new MultiIdentCompressor();
	
	private static String printNode_(Vertex V) {
		StringBuffer buffer = new StringBuffer();
		RVertexList tuple = C3PartUtil.getNodes(V);
		buffer.append("[");
		for (Vertex v : tuple) {
			buffer.append(v + " ");
		}
		buffer.append("]");
		return buffer.toString();
	}
	
	private static String getNodeType_(Vertex V) {
		if (compress_.isRoot(V))
			return "Root   ";
		if (compress_.isAlias(V))
			return "Alias  ";
		return "Normal ";
	}
	
	/*
	 * API
	 */

	/**
	 * print node
	 */
	public static void printNode(Vertex V, String prefix, boolean aliasToo) {
		System.err.println(prefix + " " + getNodeType_(V) + ": " + printNode_(V));
		if (aliasToo) {
			for (Vertex U : C3PartUtil.getAliases(V)) {
				System.err.println(prefix + " " + getNodeType_(U) + ": " + printNode_(U));
			}
		}
	}
	
	/**
	 * check colors of edge U-V
	 */
	public static int checkColors(Vertex V, Vertex U, String prefix, boolean fixme) {
		RColorSet eColors = RColorSet.getColors(V.getEdgeTo(U));
		RColorSet tColors = C3PartUtil.getPseudoColors(V, U);
		if (eColors.equals(tColors))
			return 0;
		System.err.println("+ [MultiGraphDebugger] " + prefix + 
	                       " Invalid Edge color from : " + printNode_(V) +
	                       " to : " + printNode_(U));
		System.err.println("+ [MultiGraphDebugger] " + prefix + 
				            " Edge colors : "  + eColors +
				            " Pseudo colors : " + tColors +
				            " True colors : " + C3PartUtil.getTrueColors(U, V));
		if (fixme) {
			eColors.setColors(tColors);
		}
		return 1;
	}

	/**
	 * check colors of multigraph
	 * pass null as compress to ignore compression
	 */
	public static int checkColors(Graph multi, String prefix, boolean fixme) {
		int nerr = 0;
		for (Vertex V : multi.vertices()) {
			for (Vertex U : V.neighbors()) {
				nerr += checkColors(V, U, prefix, fixme);
			}
		}
		return nerr;
	}

	/**
	 * fix colors of edge UV
	 */
	public static void fixColors(Vertex V, Vertex U) {
		RColorSet eColors = RColorSet.getColors(V.getEdgeTo(U));
		eColors.setColors(C3PartUtil.getPseudoColors(U, V));
	}

	/**
	 * fix colors of multigraph
	 */
	public static void fixColors(Graph multi) {
		for (Vertex V : multi.vertices()) {
			for (Vertex U : V.neighbors()) {
				fixColors(V, U);
			}
		}
	}
	
}
