package helix.graph.algo.c3part;

import static org.junit.Assert.assertEquals;
import helix.graph.io.DimacsExtendedReader;
import helix.graph.io.GraphReader;
import helix.graph.model.Graph;
import helix.graph.model.GraphFactory;
import helix.graph.model.impl.RColorSet;
import helix.graph.model.impl.memory.MemGraphFactory;
import helix.graph.util.LogUtil;
import helix.graph.util.StopWatch;

import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.Set;
import java.util.logging.Logger;

import org.junit.Test;

public class TestC3PartOTF {
	
	@Test
	public void testC3PartOTF() throws IOException {
		
		// ypol's paper example
		//
		String sin = "p test\nn label=a1 colors=10\nn label=a2 colors=10\nn label=a3 colors=10\nn label=a4 colors=10\nn label=a5 colors=10\n" +
					 "n label=b1 colors=100\nn label=b2 colors=100\nn label=b3 colors=100\nn label=b4 colors=100\n" +
				     "e 1 2 colors=10\ne 2 3 colors=10\ne 3 4 colors=10\ne 4 5 colors=10\n" +
				     "e 6 7 colors=100\ne 7 8 colors=100\ne 8 9 colors=100\n" +
				     "e 1 6 colors=1\ne 2 7 colors=1\ne 3 7 colors=1\ne 5 7 colors=1\ne 4 9 colors=1\n";

		StringReader sreader = new StringReader(sin);
		GraphFactory factory = MemGraphFactory.instance();
		GraphReader reader = new DimacsExtendedReader(sreader, factory);
		Graph graph = reader.read();
		
		assertEquals("graph size", 9, graph.nbVertices());
		assertEquals("nb colors", 3, RColorSet.nbColors(graph));

		C3PartOTF algo = new C3PartOTF(graph);
		C3PartOptions options = algo.getOptions();
		// options.debug.addHandler("debug");
		options.debug.setVerbose(true);
		options.debug.addVertexAttribute(C3PartUtil.Nodes);
		
		algo.setOptions(options);
		Set<Graph> part = algo.run();
		
		assertEquals("partition size", 3, part.size());

		int max = 0;
		for (Graph g : part) {
			assertEquals("nb colors", 2, RColorSet.nbColors(g));
			if (g.nbVertices() > max)
				max = g.nbVertices();
		}
		
		assertEquals("max clas size", 3, max);
	}

	@Test
	public void testC3PartOTFLarge() throws IOException {

		Logger logger = LogUtil.getLogger();
		
		StopWatch w = new StopWatch(logger);
		w.start();
		
		FileReader freader = new FileReader("samples/sample.poly");
		GraphFactory factory = MemGraphFactory.instance();
		GraphReader reader = new DimacsExtendedReader(freader, factory);
		Graph graph = reader.read();

		C3PartOTF algo = new C3PartOTF(graph);
		C3PartOptions options = algo.getOptions();
		
		w.log("reader:") ; w.reset();

		options.optimizerName = C3PartOptions.NO_OPTIMIZER;
		algo.setOptions(options);
		Set<Graph> part = algo.run();
		w.log("C3PartOTF dry:"); w.reset();

		assertEquals("partition size", 32, part.size());

		options.optimizerName = C3PartOptions.GLOBAL_OPTIMIZER;
		algo.setOptions(options);
		algo.setOptions(options);
		part = algo.run();
		w.log("C3PartOTF best:"); w.reset();

		assertEquals("partition size", 32, part.size());
		
		int max = 0;
		for (Graph clas : part) {
			if (clas.nbVertices() > max)
				max = clas.nbVertices();
		}

		assertEquals("max clas size", 116, max);

	}
}

