package helix.graph.algo.c3part;

import static org.junit.Assert.assertEquals;
import helix.graph.io.DimacsExtendedReader;
import helix.graph.io.GraphReader;
import helix.graph.model.Graph;
import helix.graph.model.GraphFactory;
import helix.graph.model.impl.RColorSet;
import helix.graph.model.impl.memory.MemGraphFactory;
import helix.graph.util.LogUtil;
import helix.graph.util.StopWatch;

import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.Set;
import java.util.logging.Logger;

import org.junit.Test;

public class TestC3PartOTFQGlobal {
	
	@Test
	public void testC3PartOTFQGlobal() throws IOException {
		
		String sin = "p test\n" +
					 "n label=a1 colors=10  \nn label=a2 colors=10  \nn label=a3 colors=10  \nn label=a4 colors=10  \nn label=a5 colors=10  \nn label=a6 colors=10  \nn label=a7 colors=10  \nn label=a8 colors=10  \n" +
					 "n label=b1 colors=100 \nn label=b2 colors=100 \nn label=b3 colors=100 \nn label=b4 colors=100 \nn label=b5 colors=100 \nn label=b6 colors=100 \nn label=b7 colors=100 \nn label=b8 colors=100 \n" +
					 "n label=c1 colors=1000\nn label=c2 colors=1000\nn label=c3 colors=1000\nn label=c4 colors=1000\nn label=c5 colors=1000\nn label=c6 colors=1000\nn label=c7 colors=1000\nn label=c8 colors=1000\n" +
				     "e  1  2 colors=10  \ne  3  4 colors=10  \ne  5  6 colors=10  \n" +
				     "e  9 10 colors=100 \ne 11 12 colors=100 \ne 15 16 colors=100 \n" +
				     "e 17 18 colors=1000\ne 19 20 colors=1000\ne 21 22 colors=1000\ne 23 24 colors=1000\n" +
				     "e  1  9 colors=1\ne  1 17 colors=1\ne  9 17 colors=1\n" +
				     "e  2 10 colors=1\ne 10 18 colors=1\n" +
				     "e  3 11 colors=1\n" +
				     "e 12 20 colors=1\n" +
				     "e 13 21 colors=1\n" +
				     "e  7 23 colors=1\n" +
				     "e 16 24 colors=1\n" +
				     "";
		StringReader sreader = new StringReader(sin);
		GraphFactory factory = MemGraphFactory.instance();
		GraphReader reader = new DimacsExtendedReader(sreader, factory);
		Graph graph = reader.read();
		
		assertEquals("graph size", 24, graph.nbVertices());
		assertEquals("nb colors", 4, RColorSet.nbColors(graph));

		C3PartOTFQGlobal algo = new C3PartOTFQGlobal(graph);
		C3PartOptions options = algo.getOptions();
		
		options.maxstar = 1;
		options.minsize = 1;
		options.mineltsize = 2;
		
		// options.colors = new IPermutation("3 2 1");
		// options.debug.addHandler("debug");
		// options.debug.setVerbose(true);
		// option.debug.addVertexAttribute(Registry.Label);
		// options.debug.addVertexAttribute(C3PartUtil.Nodes);
		
		algo.setOptions(options);
		
		Set<Graph> part = algo.run();
		
		assertEquals("partition size", 2, part.size());

		int max = 0;
		for (Graph g : part) {
			// CCCReporter.printSynton(g);
			// CCCReporter.println("");
			assertEquals("nb colors", 3, RColorSet.nbColors(g));
			if (g.nbVertices() > max)
				max = g.nbVertices();
		}
		
		assertEquals("max clas size", 2, max);
	}

	
	@Test
	public void testC3PartOTFQGlobalLarge() throws IOException {

		Logger logger = LogUtil.getLogger();
		
		StopWatch w = new StopWatch(logger);
		w.start();
		
		FileReader freader = new FileReader("samples/entero3.poly");
		GraphFactory factory = MemGraphFactory.instance();
		GraphReader reader = new DimacsExtendedReader(freader, factory);
		Graph graph = reader.read();
		
		w.log("reader:") ; w.reset();
		
		C3PartOTFQGlobal algo = new C3PartOTFQGlobal(graph);
		C3PartOptions options = algo.getOptions();
		
		options.maxstar = 1;
		options.mineltsize = 10;
		options.optimizerName = C3PartOptions.NO_OPTIMIZER;
		
		// LogUtil.setVerbose(options.logger, true);
		
		// options.debug.addHandler("debug");
		// options.debug.setVerbose(true);

		algo.setOptions(options);
		Set<Graph> part = algo.run();
		w.log("C3PartOTFQGlobal:"); w.reset();

		assertEquals("partition size", 19, part.size());
		
		int max = 0;
		for (Graph clas : part) {
			// CCCReporter.printSynton(clas);
			// CCCReporter.println("");
			if (clas.nbVertices() > max)
				max = clas.nbVertices();
		}

		assertEquals("max clas size", 29, max);

	}	
	
}

