package helix.graph.algo.c3part.compressor;

/**
 * a multigraph compressor with the following subsumption:<br>
 * <code>
 * V S U <=> Colors(U) = Colors(V) = Colors(Edge(U,V))  // @NonStrict
 *           and 
 *           Neighbors(V) = Neighbors(U)
 *           and 
 *           A W in Neighbors(V), Colors(Edge(V,W)) = Colors(Edge(U,W))
 * Note that S is an equivalence relation.
 * // @NonStrict : for OTF, the conditions reads Colors(Edge(U,V)) = AllColors<br>
 * </code>
 */

import helix.graph.model.Edge;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

public class MultiIdentCompressor extends AbstractMultiCompressor {
	
	/**
	 * internal utility
	 * check equality of colors
	 */
	private final boolean sameColors_(Edge e1, Edge e2) {
		return RColorSet.getColors(e1).equals(RColorSet.getColors(e2));
	}
	
	/**
	 * delegate patternSet
	 * @Implement
	 */
	public final Iterable<Vertex> patternSet_(Vertex V, Graph multi) {
		return V.neighbors();
	}
	
	/**
	 * delegate subsumption
	 * @Implement
	 */
	public boolean subsumes_(Vertex V, Vertex U, Graph multi) {
		for (Vertex W : V.neighbors()) {
			if (W == U)
				continue;
			Edge UW = U.getEdgeTo(W);
			if (UW == null)
				return false;
			if (! sameColors_(UW, V.getEdgeTo(W)))
				return false;
		}
		for (Vertex W : U.neighbors()) {
			if (W == V)
				continue;
			Edge VW = V.getEdgeTo(W);
			if (VW == null)
				return false;
		}
		return true;
	}
	
	/**
	 * delegate restoration
	 * restore (if needed) edge between V and W 
	 * @Implement
	 */
	public void restoreEdge_(Vertex V, Vertex W, Vertex Root, Graph multi) {
		Edge E = multi.addEdge(V, W);
		E.copy(Root.getEdgeTo(W)); // restore root edge attributes
	}
}
