package helix.graph.algo.c3part.compressor;

import java.util.logging.Logger;

import helix.graph.algo.c3part.C3PartUtil;
import helix.graph.model.Edge;
import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

/**
 * A compressor that does nothing
 */

public class NoCompressor implements MultiGraphCompressor {

	/*
	 * API
	 */
	
	public boolean subsumes(Vertex v, Vertex u, Graph graph) {
		return false;
	}

	public void compressVertex(Vertex v, Vertex root, Graph graph) {
	}
	
	public void compress(Graph graph) {
	}

	public void compress(Graph graph, Logger logger) {
	}

	public void expandVertex(Vertex v, Graph graph) {
	}

	public void expand(Graph graph) {
	}

	public Iterable<Vertex> getVertices(Graph graph) {
		return graph.vertices();
	}

	public int nbVertices(Graph graph) {
		return graph.nbVertices();
	}

	public Iterable<Vertex> getNeighbors(Vertex v) {
		return v.neighbors();
	}

	public RColorSet getColors(Vertex V, Vertex U) {
		Edge E = V.getEdgeTo(U);
		return (E != null ? RColorSet.getColors(E) : C3PartUtil.getTrueColors(V, U));
	}
	
	public boolean isAlias(Vertex V) {
		return false;
	}

	public boolean isRoot(Vertex V) {
		return false;
	}

	public boolean isNormal(Vertex V) {
		return true;
	}
	
	public Vertex getRoot(Vertex V) {
		return null;
	}
	
	public Iterable<Vertex> getAliases(Vertex V) {
		return null;
	}

	public void flatten(Graph graph) {
		GSet<Vertex> vertices = graph.vertexSetFactory().make(graph.vertices());
		for (Vertex v : vertices) {
			graph.disconnectVertex(v);
		}
	}
	
	
	public void storeSet(GSet<Vertex> set) {
	}

	public GSet<Vertex> retrieveSet() {
		return null;
	}

	public String getParamInfo() {
		return "";
	}
}
