package helix.graph.algo.c3part.lookahead;

/**
 * a lookahead connector providing the connected components of current terminals
 * following only links of current color
 */

import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

public class ColorPathConnector extends TerminalsConnector {

	protected static LookaheadConnector instance_ = new ColorPathConnector();

	/**
	 * singleton class
	 */
	protected ColorPathConnector() {
	}

	public static LookaheadConnector getInstance() {
		return instance_;
	}

	/**
	 * utility: get the connected component of datanode v
	 * restricted to nodes of given color by following only links of that color.
	 */
	private final void colorPathConnected_(Vertex v, int color, GSet<Vertex> store) {
		if (store.contains(v))
			return;
		store.add(v);
		for (Vertex u : v.neighbors()) {
			if (RColorSet.hasColor(u, color))
				colorPathConnected_(u, color, store);
		}
	}

	/**
	 * get the connected component of all datanodes in multigraph terminals
	 * following only links of terminal color.
	 * store them into 'store'
	 */
	@Override
	public void getLookAhead(Graph datagraph, Graph multigraph, int color, GSet<Vertex> store) {
		GSet<Vertex> terminals = getConnectors(multigraph);
		for (Vertex v : terminals)
			colorPathConnected_(v, color, store);
	}

}
