package helix.graph.algo.c3part.lookahead;

/**
 * a lookahead connector that returns all datanodes of given color
 * in current datagraph (i.e. whatever the current multigraph is)
 */

import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

public class FullColorConnector extends LookaheadConnector {

	protected static LookaheadConnector instance_ = new FullColorConnector();

	/**
	 * singleton class
	 */
	protected FullColorConnector() {
	}

	public static LookaheadConnector getInstance() {
		return instance_;
	}
	
	/**
	 * @Implement
	 * does nothing
	 */
	protected GSet<Vertex> getConnectors(Graph multigraph) {
		return null;
	}

	/**
	 * @Implement
	 * just store all datanodes of given color into 'store'
	 */
	public void getLookAhead(Graph datagraph, Graph multigraph, int color, GSet<Vertex> store) {
		for (Vertex v : datagraph.vertices()) {
			if (RColorSet.hasColor(v, color))
				store.add(v);
		}
	}

}
