package helix.graph.algo.c3part.lookahead;

/**
 * a lookahead connector to get the connected component of *all datanodes* in
 * current multigraph (i.e. not only terminals) 
 * restricted to nodes of given color and following the one-way path.
 * The 'one-way' path is simpler variant of the 'two-ways' path.
 * TODO: please add some explanation here
 */

import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

public class OneWayPathConnector extends ExtendedConnector {

	protected static LookaheadConnector instance_ = new OneWayPathConnector();

	/**
	 * singleton class
	 */
	protected OneWayPathConnector() {
	}

	public static LookaheadConnector getInstance() {
		return instance_;
	}

	/**
	 * get datanodes of color 'color' that are connected to v by following
	 * a 'one-way' path.
	 * A 'one-way' path is simpler variant of the 'two-ways' path.
	 * TODO: please add some explanation here
	 * 
	 * note: improvement : forbidColors : set of colors that can be skipped during a pending
	 * (or after ?) path. may be empty for tests
	 * 
	 */
	protected void oneWayConnected_(Vertex v, int color, RColorSet forbidColors, 
									GSet<Vertex> visited, GSet<Vertex> store) {
		if (visited.contains(v))
			return;
		visited.add(v);
		int vcolor = RColorSet.getColors(v).singleColor();
		if (vcolor == color)
			store.add(v);
		if (forbidColors.hasColor(vcolor))
			return;
		for (Vertex u : v.neighbors()) {
			RColorSet newForbid = forbidColors;
			int ucolor = RColorSet.getColors(u).singleColor();
			if (ucolor != vcolor) { // epsilon link
				newForbid = new RColorSet(forbidColors);
				newForbid.setColor(vcolor); // forbid v
			}
			oneWayConnected_(u, color, newForbid, visited, store);
		}
	}

	/**
	 * @Implement
	 * get the connected component of all datanodes in multigraph tuples
	 * restricted to nodes of given color and following a one-way path.
	 * A 'one-way' path is simpler variant of the 'two-ways' path.
	 * TODO: please add some explanation here
	 * store them into 'store'
	 */
	public void getLookAhead(Graph datagraph, Graph multigraph, int color, GSet<Vertex> store) {
		GSet<Vertex> connectors = getConnectors(multigraph);
		GSet<Vertex> visited = datagraph.vertexSetFactory().make();
		RColorSet forbidColors = new RColorSet(RColorSet.getColors(multigraph));
		forbidColors.clearColor(color);
		for (Vertex v : connectors) {
			oneWayConnected_(v, color, forbidColors, visited, store);
		}
	}
	
}
