package helix.graph.algo.c3part.lookahead;

/**
 * a lookahead connector that merely returns the current terminals
 * of the multigraph (except stars)
 */

import helix.graph.algo.c3part.C3PartUtil;
import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

public class TerminalsConnector extends LookaheadConnector {

	protected static LookaheadConnector instance_ = new TerminalsConnector();

	/**
	 * singleton class
	 */
	protected TerminalsConnector() {
	}

	public static LookaheadConnector getInstance() {
		return instance_;
	}

	/**
	 * @Implement
	 * connectors are current terminals of multigraph (except stars)
	 */
	protected GSet<Vertex> getConnectors(Graph multigraph) {
		GSet<Vertex> connectors = multigraph.vertexSetFactory().make();
		for (Vertex V : multigraph.vertices()) {
			Vertex v = C3PartUtil.getLastNode(V);
			if (! RColorSet.getColors(v).isEmpty()) { // i.e no star
				connectors.add(v);
			}
		}
		return connectors;
	}
	
	/**
	 * @Implement
	 * just store current terminals into 'store'
	 */
	public void getLookAhead(Graph datagraph, Graph multigraph, int color,GSet<Vertex> store) {
		GSet<Vertex> terminals = getConnectors(multigraph);
		store.addAll(terminals);
	}

}
