package helix.graph.algo.c3part.optimizer;

import java.util.ArrayList;
import java.util.List;

public abstract class AbstractColorOptimizer implements ColorOptimizer {

	protected ArrayList<Integer> usedColors_ = new ArrayList<Integer>();
	
	protected boolean giveup_ = false;
	
	/*
	 * required API
	 */

	/**
	 * remove not implemented : throws UnsupportedOperationException
	 */
	public void remove() {
		throw new UnsupportedOperationException();
	}

	/**
	 * get already provided colors
	 */
	public List<Integer> getUsedColors() {
		return new ArrayList<Integer>(usedColors_);
	}
	
	/**
	 * return the rank (starting at 0) at which the 
	 * specified color was produced 
	 * i.e. the index of color in  getUsedColors
	 */
	public int getColorIndex(int color) {
		return usedColors_.indexOf(color);
	}

	/**
	 * don't do anything on giveup : just keep track of this call
	 */
	public void giveup() {
		giveup_ = true;
	}

	/**
	 * default preview : no preview available
	 */
	public String preview() {
		return "(no preview)";
	}

	/**
	 * default optional parameters info : no info
	 */
	public String getParamInfo() {
		return "";
	}

	/*
	 * added API
	 */
	/**
	 * store currently provided  color
	 */
	protected void storeColor_(int color) {
		usedColors_.add(color);
	}
	
	@Override
	/**
	 * a human readable version of this optimizer that shows used colors
	 */
	public String toString() {
		StringBuffer buf = new StringBuffer(super.toString() + " used colors = ");
		for (int c : this.getUsedColors())
			buf.append(c + " ");
		return buf.toString();
	}
	
}
