package helix.graph.algo.c3part.optimizer;

import java.util.Iterator;
import java.util.List;

public interface ColorOptimizer extends Iterator<Integer> {

	/**  
	 * return a (copy) list of already provided colors.
	 * 
	 * note: the interpretation of 'already provided' may be different
	 * for each optimizer (see notes in subclasses).
	 * by default it means the colors that have been
	 * previously produced by each call to next(). 
	 */
	public abstract List<Integer> getUsedColors();
	
	
	/**
	 * return the rank (starting at 0) at which the 
	 * specified color was produced 
	 * this is formally equivalent to <code>getUsedColors().IndexOf(color)</code>
	 * but may be quicker in some cases
	 */
	public abstract int getColorIndex(int color);
	
	/**
	 * experimental section: for gc
	 * this method may be called when the optimizer will never be used again
	 * it will allow to release some internal field for the garbage collector.
	 */
	public abstract void giveup();
	
	/**
	 * give a preview of this optimizer colors if possible
	 * this is used only for logging purpose (for user information)
	 * so you may just return an empty string
	 */
	public abstract String preview();

	/**
	 * get optional parameters info
	 * this is used only for logging purpose (for user information)
	 * so you may just return an empty string
	 */
	public abstract String getParamInfo();
	
}
