package helix.graph.algo.c3part.optimizer;

import helix.graph.algo.c3part.C3PartAlgo;
import helix.graph.algo.c3part.C3PartUtil;
import helix.graph.algo.c3part.reporter.GraphStatistics;
import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;

public class TerminalsStarColorOptimizer extends TerminalsColorOptimizer {

	protected int maxStar_;
	
	public TerminalsStarColorOptimizer(C3PartAlgo algo, Graph datagraph, Graph clas) {
		super(algo, datagraph, clas);
		maxStar_ = algo.getPhysicalOptions().maxstar;
	}
	
	@Override
	/**
	 * utility : compute size of color terminals
	 */
	protected int getSizeTerminals_(int color) {

		GSet<Vertex> terminals = datagraph_.vertexSetFactory().make();
		
		boolean isStarExpandable = false;
		
		for (Vertex V : clasVertices_) {

			// count stars in V nodes
			//
			int countStar = 0;
			for (Vertex v : C3PartUtil.getNodes(V)) {   // foreach node v in tuple
				if (C3PartUtil.isStar(v))
					countStar++;
			}
			
			// check for black star (Clas is a singleton)
			// in that case terminals will be the whole genome 'color'
			//
			if (countStar == C3PartUtil.getNodes(V).size()) {
				return GraphStatistics.verticesPerColor(datagraph_)[color];
			}
			
			if (countStar < maxStar_) {
				isStarExpandable = true;
			}
			
			terminals.addAll(aggregator_.getTerminals(V, color));
		}
		
		if (terminals.size() != 0)
			return terminals.size();  // no problem
		
		return (isStarExpandable ? Integer.MAX_VALUE : 0); // avoid star expansion
		
	}
	
	
	

}
