package helix.graph.algo.c3part.preprocessor;

import helix.graph.algo.CCVisitor;
import helix.graph.algo.c3part.visitor.CCByEpsilonVisitor;
import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

/**
 * a class to remove vertices of a datagraph
 * that belong to partially colored epsilon CC.
 * a partially colored epsilon CC has a total number
 * of color less than the specified threshold.
 */
public class CCColorCleaner extends DatagraphCleaner {

	protected int mincolors_;
	
	protected RColorSet setColors_ = new RColorSet();
	
	/**
	 * compute colors of set
	 */
	protected final RColorSet getSetColors_(GSet<Vertex> set) {
		setColors_.clearColors();
		for (Vertex v : set) {
			setColors_.orColors(RColorSet.getColors(v));
		}
		return setColors_;
	}

	/**
	 * constructor: specify minimum number of colors
	 */
	public CCColorCleaner(int mincolors) {
		mincolors_ = mincolors;
	}
	
	/**
	 * @Implements
	 */
	public void run(Graph graph) {
		// -1- compute epsilon CC
		CCVisitor visitor = new CCByEpsilonVisitor(graph);
		Iterable<GSet<Vertex>> ccs = visitor.traverse().getComponents();
		
		// -2- remove vertices
		
		RColorSet allColors = new RColorSet(RColorSet.getColors(graph));
		allColors.clearColor(0);

		for (GSet<Vertex> cc : ccs) {
			int ncolors = getSetColors_(cc).countColors();
			if (ncolors < mincolors_) {
				for (Vertex v : cc) {
					graph.removeVertex(v);
				}
			}
		}
	}
		


}
