package helix.graph.algo.c3part.preprocessor;

/**
 * this cleaner remove vertices with a pivot color condition:
 * if vertex v has the pivot color : then it should be epsilon connected to at least quorum
 * different colors (including its own color)
 * else it should be connected to at least one vertex of the pivot color
 */

import helix.graph.model.Registry;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

public class CenteredQuorumCleaner extends QuorumCleaner {

	protected int pivot_;

	/*
	 * internal utility : (almost) safely get single color
	 */
	private static final int color_(Registry r) {
		return (RColorSet.hasColors(r) ? RColorSet.getColors(r).singleColor() : -1);
	}

	public CenteredQuorumCleaner(int quorum, int pivot) {
		super(quorum);
		pivot_ = pivot;
	}

	@Override
	/**
	 * check epsilon pivot quorum for vertex v
	 * if vertex v has the pivot color : then it should be epsilon connected to at least quorum
	 * different colors (including its own color)
	 * else it should be connected to at least one vertex of the pivot color
	 */ 
	protected boolean validate(Vertex v) {
		
		if (color_(v) == pivot_)
			return super.validate(v); // requires quorum
		
		for (Vertex u : v.neighbors()) {
			if (color_(u) == pivot_)
				return true;
		}
		
		return false;
	}

	@Override
	public void logParams() {
		super.logParams();
		log("pivot : " + pivot_);
	}

}
