package helix.graph.algo.c3part.preprocessor;

import java.util.HashSet;

import helix.graph.model.Edge;
import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Registry;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

/**
 * a class to remove some colors from datagraph
 * and shift remaining colors accordingly
 */
public class ColorCleaner extends DatagraphCleaner {

	protected HashSet<Integer> keepColors_ = new HashSet<Integer>();

	public ColorCleaner(Iterable<Integer> keepColors) {
		for (int color : keepColors)
			keepColors_.add(color);
	}

	/*
	 * internal utility : (almost) safely get single color
	 */
	private static final int color_(Registry r) {
		return (RColorSet.hasColors(r) ? RColorSet.getColors(r).singleColor() : 0);
	}
	
	public void run(Graph graph) {
		// -1- remove color vertices
		//
		GSet<Vertex> removed = graph.vertexSetFactory().make();
		for (Vertex v : graph.vertices()) {
			if (! keepColors_.contains(color_(v)))
				removed.add(v);
		}
		for (Vertex v : removed) {
			graph.removeVertex(v);
		}
		// -2- shift remaining colors
		//
		// get correspondance
		//
		int corresp[] = new int[RColorSet.nbColors(graph)];
		int newColor = 1;
		for (int color = 1 ; color < corresp.length ; color++) {
			if (keepColors_.contains(color)) {
				corresp[color] = newColor++;
			}
		}
		// apply correspondance
		//
		GSet<Edge> edges = graph.edgeSetFactory().make();
		RColorSet cset;
		for (Vertex v : graph.vertices()) {
			int vcolor = color_(v);
			cset = RColorSet.getColors(v);
			cset.clearColor(vcolor);
			cset.setColor(corresp[vcolor]);
			for (Edge e : v.edges()) {
				if (edges.contains(e))
					continue;
				cset = RColorSet.getColors(e);
				for (int ecolor : new RColorSet(cset)) {
					cset.clearColor(ecolor);
					cset.setColor(corresp[ecolor]);
				}
				edges.add(e);
			}
		}
		// change graph colors
		//
		cset = RColorSet.getColors(graph);
		for (int color = 1 ; color < corresp.length ; color++) {
			cset.clearColor(color);
			if (corresp[color] != 0)
				cset.setColor(corresp[color]);
		}
	}

	@Override
	public void logParams() {
		super.logParams();
		log("keepColors : " + keepColors_.toString());
	}

}
