package helix.graph.algo.c3part.preprocessor;

import java.util.ArrayList;

import helix.graph.algo.DeltaClosure;
import helix.graph.model.Edge;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

/**
 * a class to compute delta closure on the epsilon (correspondance) 
 * relation in datagraphs.<br>
 * note: to close on other (non zero color) relations, use the
 * <code>DataGraphCloser</code> class instead.
 */

public class EpsilonCloser extends GraphCloser {

	/**
	 * internal constraint to close on color 0 (epsilon relation)
	 */
	private class EpsilonConstraint extends DeltaClosure.Constraint {
		public boolean accept(Edge e) {
			// accept epsilon edge
			int color1 = RColorSet.getColors(e.v1()).singleColor();
			int color2 = RColorSet.getColors(e.v2()).singleColor();
			return (color1 != color2);
		}
		public boolean validate(Vertex v1, Vertex v2) {
			int color1 = RColorSet.getColors(v1).singleColor();
			int color2 = RColorSet.getColors(v2).singleColor();
			return (color1 != color2);
		}
		public void register(Edge e) {
			RColorSet.getColors(e).setColor(0);
		}
	}

	/**
	 * API
	 */
	
	/**
	 * close on color 0 (epsilon) with delta.
	 * delta < 0 => full transitive closure
	 */
	public EpsilonCloser(int delta) {
		delta_ = new ArrayList<Integer>(1);
		delta_.add(delta);
	}

	/**
	 * implement our run() operation
	 */
	//@Implement
	public void run(Graph graph) {

		int delta = delta_.get(0) + 1; // +1 for standard interpretation
		
		if (delta <= 0) 
			throw new PreprocessorException("warshall is unfinished, blame developers...");
		
		//if (delta == 1)
		//	return;
		
		DeltaClosure.Constraint constraint = new EpsilonConstraint();
		DeltaClosure closure = new DeltaClosure(graph, delta, constraint);
		closure.run();
	}
}
