package helix.graph.algo.c3part.preprocessor;

/**
 * an abstract root class for datagraph and multigraph
 * delta closure computations.
 */
import java.util.ArrayList;
import java.util.List;

public abstract class GraphCloser extends GraphPreprocessor {

	/**
	 * the internal copy of delta list
	 */
	protected List<Integer> delta_ = null;
	
	/**
	 * internal utility: 
	 * check if deltaList is composed of same element
	 */
	protected static final boolean sameDelta_(List<Integer> deltaList) {
		int delta0 = deltaList.get(0);
		for (int delta : deltaList) {
			if (delta != delta0)
				return false;
		}
		return true;
	}

	/**
	 * internal utility:
	 * initialize internal delta list
	 */
	protected static final List<Integer> initDelta_(List<Integer> deltaList) {
		if ((deltaList == null) || (deltaList.size() == 0))
			throw new PreprocessorException("delta list should be at least of size 1");
		return new ArrayList<Integer>(deltaList);
	}

	/**
	 * internal utility: make a delta list of given size<br>
	 * if delta.size == 1 -> assume same delta for all elements, and make a list of
	 * size element equals to delta[0]<br>
	 * if delta.size >= size -> copy delta truncated to size 
	 * if delta.size < size -> throw exception<br>
	 */
	protected static final List<Integer> makeDelta_(List<Integer> deltaList, int size) {
		
		int deltaSize = deltaList.size();
		
		if ((deltaSize != 1) && (deltaSize < size))
			throw new PreprocessorException("delta list should be at least of size " + size);
		
		ArrayList<Integer> deltaNew = new ArrayList<Integer>(size);
		
		for (int i = 0 ; i < size ; i++) {
			int deltai = deltaList.get(deltaSize == 1 ? 0 : i);
			deltaNew.add(deltai);
		}
		
		return deltaNew;
	}

	/**
	 * API
	 */

	/**
	 * make a delta list of given size<br>
	 * if delta.size == 1 -> assume same delta for all elements, and make a list of
	 * size element equals to delta[0]<br>
	 * if delta.size >= size -> copy delta truncated to size 
	 * if delta.size < size -> throw exception<br>
	 */
	public static List<Integer> makeDeltaList(List<Integer> delta, int size) {
		return makeDelta_(initDelta_(delta), size);
	}
	
	
	/**
	 * override logParams
	 */
	@Override
	public void logParams() {
		super.logParams();
		log("delta : " + delta_.toString());
	}
}
