package helix.graph.algo.c3part.preprocessor;

/**
 * this generic cleaner examine each vertex in turn and call internal <code>validate(Vertex v)</code>
 * to remove it or not. The process is iterated until no change occurs in graph.
 */

import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;

public abstract class IterativeCleaner extends DatagraphCleaner {

	protected abstract boolean validate(Vertex v);
	
	public void run(Graph graph) {
		GSet<Vertex> vertices = graph.vertexSetFactory().make(graph.vertices());
		GSet<Vertex> removed = graph.vertexSetFactory().make();
		do {
			removed.clear();
			for (Vertex v : vertices) {
				if (! validate(v)) {
					graph.removeVertex(v);
					removed.add(v);
				}
			}
			vertices.removeAll(removed);
		} while (removed.size() != 0);
	}

}
