package helix.graph.algo.c3part.preprocessor;

/**
 * this cleaner remove vertices that don't have epsilon links to at least quorum
 * different colors (including their own color)
 */

import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;

public class QuorumCleaner extends IterativeCleaner {

	// to speedup isValid_() : avoid constructing a RColorSet each time
	private final RColorSet colors_ = new RColorSet();

	protected int quorum_;

	public QuorumCleaner(int quorum) {
		quorum_ = quorum;
	}

	/**
	 * check epsilon quorum for vertex v
	 * v should be epsilon connected to at least quorum_
	 * different colors (including its own color)
	 */ 
	protected boolean validate(Vertex v) {
		colors_.clearColors();
		colors_.orColors(RColorSet.getColors(v));
		for (Vertex u : v.neighbors()) {
			colors_.orColors(RColorSet.getColors(u));
		}
		return (colors_.countColors() >= quorum_);
	}

	@Override
	public void logParams() {
		super.logParams();
		log("quorum : " + quorum_);
	}
	
}
