package helix.graph.algo.c3part.reporter;

import helix.graph.algo.c3part.C3PartOptions;
import helix.graph.algo.c3part.C3PartUtil;
import helix.graph.model.Graph;
import helix.graph.model.Registry;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RColorSet;
import helix.graph.model.impl.RVertexList;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

/**
 * a clas to report CCCs in Dimacs-like format :
 * 
 * q #vertices #node1 ... #nodesN
 * v name1... nameN rank1 ... rankN | commment1 | ... | commentN
 * v ...
 * 
 */

public class CCCReporter extends BaseReporter {

	/*
	 * get number of different datagraph vertices in synton
	 */
	protected static List<Integer> getNbElements_(Graph synton) {

		// prepare hashtables
		ArrayList<HashSet<Vertex>> listHash = new ArrayList<HashSet<Vertex>>();
		
		int nbColor = RColorSet.nbColors(synton);
		for (int i = 0 ; i < nbColor ; i++) {
			listHash.add(new HashSet<Vertex>());
		}

		// hash datagraph vertices
		for (Vertex V : synton.vertices()) {

			RVertexList tuple = C3PartUtil.getNodes(V);
			
			for (int i = 0 ; i < tuple.size() ; i++) {
				HashSet<Vertex> h = (HashSet<Vertex>) listHash.get(i);
				Vertex v = tuple.getVertex(i);
				if (C3PartUtil.isStar(v)) continue;
				h.add(tuple.getVertex(i));
			}
		}
		
		// count occurences
		ArrayList<Integer> res = new ArrayList<Integer>();
		for (HashSet<Vertex> h : listHash) {
			res.add(h.size());
		}
		return res;
	}
	
	/*
	 * print a single multinode
	 */
	protected static void printNode_(Vertex v) {
		RVertexList tuple = C3PartUtil.getNodes(v);
		for (Vertex vv : tuple) {
			String name = getRegistryValue_(vv, Registry.Label, "ERROR");
			print(" " + name);
		}
		for (Vertex vv : tuple) {
			String rank = getRegistryValue_(vv, Registry.Rank, "0");
			print(" " + rank);
		}
		for (Vertex vv : tuple) {
			String comment = getRegistryValue_(vv, Registry.Comment, " ");
			print(" | " + comment);
		}
	}
	
	/*
	 * API
	 */
	
	/**
	 * print out a synton in Dimacs format
	 */
	
	public static void printSynton(Graph synton, C3PartOptions options) {
		
		// q #vertices #node1 ... #nodesN
		print("q " + synton.nbVertices());
		for (int i : getNbElements_(synton)) {
			print(" " + i);
		}
		println("");

		// v name1... nameN rank1 ... rankN | commment1 | ... | commentN
		for (Vertex v : synton.vertices()) {
			print("v");
			printNode_(v);
			println("");
		}

	}

}
