package helix.graph.algo.c3part.reporter;

import helix.graph.algo.c3part.C3PartOptions;
import helix.graph.algo.c3part.C3PartUtil;
import helix.graph.algo.c3part.compressor.GraphCompressor;
import helix.graph.model.Graph;
import helix.graph.model.Registry;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RVertexList;

/**
 * a temporary debug extension of CCCReporter
 * 
 * a clas to report CCCs in Dimacs-like format :
 * 
 * q #vertices #node1 ... #nodesN
 * v name1... nameN rank1 ... rankN | commment1 | ... | commentN
 * v ...
 * 
 */

public class CCCReporterDebug extends CCCReporter {

	/*
	 * override printNode_
	 */
	protected static void printNode_(Vertex v) {
		RVertexList tuple = C3PartUtil.getNodes(v);
		print("[");
		for (Vertex vv : tuple) {
			String name = getRegistryValue_(vv, Registry.Label, "ERROR");
			print(" " + name);
		}
		print("]");
	}

	/*
	 * API
	 */
	
	/*
	 * override printSynton 
	 */
	public static void printSynton(Graph synton, C3PartOptions options) {

		GraphCompressor compress = options.compressor;
		
		// q #vertices #node1 ... #nodesN
		print("q " + synton.nbVertices());
		for (int i : getNbElements_(synton)) {
			print(" " + i);
		}
		println("");

		// v name1... nameN rank1 ... rankN | commment1 | ... | commentN
		for (Vertex v : compress.getVertices(synton)) {
			print("v");
			if (compress.isRoot(v))
				print(" Root ");
			else if (compress.isAlias(v))
				print(" Alias ");
			else
				print(" Normal ");
			
			printNode_(v);
			
			if (compress.isRoot(v)) {
				print(" Aliases ");
				for (Vertex u : C3PartUtil.getAliases(v)) {
					printNode_(u);
				}
			}

			if (compress.isAlias(v)) {
				print(" Root ");
				printNode_(compress.getRoot(v));
			}
			
			println("");
		}

	}
	
}
