package helix.graph.algo.c3part.visitor;

/**
 * a Visitor to compute overlapping star connected components (CCStar) of
 * a multigraph :
 * 
 * -1- keep red stars apart (red star is a vertex having a star on specified color)
 * -2- compute normal CCs
 * -3- foreach cc put back red stars  (if there is no CC, then return red stars)
 * 
 */

import helix.graph.algo.c3part.C3PartUtil;
import helix.graph.model.GSet;
import helix.graph.model.Graph;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RVertexList;


public class CCStarByColorVisitor extends CCByColorVisitor {

	protected Vertex starVertex_;
	protected GSet<Vertex> redStars_;
	protected int colorIndex_;

	public CCStarByColorVisitor(Graph multigraph, int color, int colorIndex, Vertex star) {
		super(multigraph, color);
		starVertex_ = star;
		colorIndex_ = colorIndex;
		redStars_ = multigraph.vertexSetFactory().make();
	}
	
	/*
	 * utility : check if tuple is a black star
	 */
	protected final boolean isBlackStar_(RVertexList tuple) {
		for (Vertex v : tuple) {
			if (v != starVertex_)
				return false;
		}
		return true;
	}

	@Override
	// TODO add @Override all over the code
	// TODO check if without braces
	/*
	 * override to keep redStars apart
	 */
	public void visit(Vertex V) {
		RVertexList tuple = C3PartUtil.getNodes(V);
		if (isBlackStar_(tuple)) {	// the black star is not red !
			super.visit(V);			// and it always comes alone ...
		}
		else if (tuple.getVertex(colorIndex_) == starVertex_) {
			redStars_.add(V);
		}
		else {
			super.visit(V);
		}
	}

	/*
	 * added api
	 */
	@Override
	public Iterable<GSet<Vertex>> getComponents() {

		// put back redstars into cc
		//
		for (GSet<Vertex> s : cc_)
			s.addAll(redStars_);
		
		// if cc is empty the we should return the red stars
		//
		if ((cc_.size() == 0) && (redStars_.size() != 0))
			cc_.add(redStars_);
		
		return cc_;
	}
}
