package helix.graph.io;

import java.io.IOException;
import java.io.Reader;
import java.util.logging.Logger;

import helix.graph.model.Graph;
import helix.graph.model.GraphFactory;
import helix.graph.util.LogUtil;

public abstract class AbstractGraphReader implements GraphReader {

	protected Reader reader_;
	protected GraphFactory factory_;
	protected Logger logger_ = LogUtil.getLogger();
	
	/**
	 * default mandatory constructor
	 */
	public AbstractGraphReader(Reader reader, GraphFactory factory) {
		reader_ = reader;
		factory_ = factory;
	}
	
	/**
	 * constructor with logging addition
	 */
	public AbstractGraphReader(Reader reader, GraphFactory factory, Logger logger) {
		this(reader, factory);
		logger_ = logger;
	}
	
	/*
	 * some internal utilities
	 */
	
	protected void severe_(String msg) throws IOException {
		logger_.severe("*Error* " + msg);
		throw new IOException(msg);
	}

	protected void warning_(String msg) {
		logger_.warning("*Warning* " + msg + " (this will be ignored)");
	}

	protected void info_(String msg) {
		logger_.info(msg);
	}
	
	/*
	 * API
	 */

	public Logger getLogger() {
		return logger_;
	}
	
	public abstract Graph read() throws IOException;

}
