package helix.graph.model;

import java.util.Set;

public interface Graph extends Registry {

	public int nbVertices();
	public int nbEdges();

	public Iterable<Vertex> vertices();

	public boolean hasVertex(Vertex v);
	public Vertex addVertex();
	public void removeVertex(Vertex v) throws NoSuchVertexException;
	public void disconnectVertex(Vertex v) throws NoSuchVertexException;
	
	public Vertex findVertex(String registryKey, RegistryValue registryValue)
	  throws DuplicateValueException;

	public Vertex findVertex(String registryKey, Object anyValue)
	  throws DuplicateValueException;

	public GSet<Vertex> findVertices(String registryKey, RegistryValue registryValue);

	public GSet<Vertex> findVertices(String registryKey, Object anyValue);

	public boolean hasEdge(Vertex v1, Vertex v2) throws NoSuchVertexException;
	public Edge getEdge(Vertex v1, Vertex v2) throws NoSuchVertexException;
	public Edge addEdge(Vertex v1, Vertex v2) throws NoSuchVertexException, SameVertexException, DuplicateEdgeException;
	public Edge assertEdge(Vertex v1, Vertex v2) throws NoSuchVertexException, SameVertexException;
	public void removeEdge(Vertex v1, Vertex v2) throws NoSuchVertexException;
	public void removeEdge(Edge e) throws NoSuchVertexException;
	
	public void clear();

	public GraphFactory factory();
	
	public GSetFactory<Vertex> vertexSetFactory();

	public GSetFactory<Edge> edgeSetFactory();
	
	/*
	 * experimental section
	 */
	
	public Set<Graph> physicalSplit(Iterable<GSet<Vertex>> splits);
	
	public void addPhysicalVertex(Vertex v);
}
