package helix.graph.model;


/**
 * a graph vertex
 * @author viari
 */

public interface Vertex extends GraphElement {
	
	/**
	 * get the number edges that are incident edges to this vertex
	 * @return vertex degree
	 */
	public int degree();
	
	/**
	 * get the edge between this vertex and destination vertex v (if it exists)
	 * @param v destination vertex
	 * @return edge (this, v) if it exists else return null
	 */
	public Edge getEdgeTo(Vertex v);

	/**
	 * tells if there is an edge between this vertex and destination vertex v (if it exists).
	 * same as <code>getEdgeTo(v) != null</code>
	 * @param v destination vertex
	 * @return true if edge (this, v) exists
	 */
	public boolean hasEdgeTo(Vertex v);

	/**
	 * get Iterable over all edges that are incident to this vertex
	 * @return Iterable<Edge>
	 */
	public Iterable<Edge> edges();
	
	/**
	 * get Iterable over all vertices that are adjacent to this vertex<br>
	 * note: the following codes are equivalent : <br>
	 * <code>
	 * for (Vertex w : v.neighbors())<br>
	 * &nbsp;&nbsp;something(w);<br>
	 * </code>
	 * and :<br>
	 * <code>  
	 * for (Edge e : v.edges()) {<br>
	 * &nbsp;&nbsp;Vertex w = e.getVertexTo(v);<br>
	 * &nbsp;&nbsp;something(w);<br>
	 * }<br>
	 * </code>
	 * @return Iterable<Vertex>
	 */
	public Iterable<Vertex> neighbors();
}
