package helix.graph.model.impl;

import helix.graph.model.IllegalValueException;
import helix.graph.model.RegistryValue;
import helix.graph.model.Vertex;

/**
 * a memory implementation of RegistryValue for arrays (fixed size tuples) of Vertices
 * @author viari
 *
 */
public class RVertexArray implements RegistryValue {

	protected Vertex array_[];
		
	// API

	// constructor make a tuple of size n
	public RVertexArray(int size) {
		this.array_ = new Vertex[size];
	}

	// clone : copy the internal array of vertices
	//         but not the vertices themselves
	//
	public RegistryValue klone() {
		RVertexArray clone = new RVertexArray(array_.length);
		for (int i = 0 ; i < array_.length ; i++)
			clone.array_[i] = array_[i];
		return clone;
	}

	public String serialize() {
		throw new IllegalValueException();
	}

	public boolean toBoolean() throws IllegalValueException {
		throw new IllegalValueException();
	}

	public float toFloat() throws IllegalValueException {
		throw new IllegalValueException();
	}

	public int toInt() throws IllegalValueException {
		throw new IllegalValueException();
	}
	
	// equals and hashcode
	
	public boolean equals(Object o) {
		return (  (o instanceof RVertexArray)
			    ? this.array_.equals(((RVertexArray) o).array_)
				: false);
	}

	public int hashCode() {
		return this.array_.hashCode();
	}

	// Iterable API
	// TODO ArrayIterator (see AdaptedIterator)
//	public Iterator<Vertex> iterator() {
//		return this.list_.iterator();
//	}

	// added functions

	public Vertex getVertex(int index) throws IndexOutOfBoundsException {
		return this.array_[index];
	}

	public Vertex setVertex(int index, Vertex v) throws IndexOutOfBoundsException {
		Vertex previous = this.array_[index];
		this.array_[index] = v;
		return previous;
	}
	
	public int size() {
		return this.array_.length;
	}

	/**
	 * a human readable version of toString()
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		for (Vertex v : this.array_) {
			buffer.append(v.toString() + ", ");
		}
		int n = (buffer.length() > 2 ? buffer.length() - 2 : 0);
		return "[" + buffer.substring(0, n).trim() + "]";
	}
	
}
