package helix.graph.model.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import helix.graph.model.IllegalValueException;
import helix.graph.model.Registry;
import helix.graph.model.RegistryValue;
import helix.graph.model.Vertex;

/**
 * a memory implementation of RegistryValue for lists of Vertices
 * @author viari
 *
 */
public class RVertexList implements RegistryValue, Iterable<Vertex> {

	protected ArrayList<Vertex> list_;
		
	// API

	//  make a list of initial capacity n
	public RVertexList(int size) {
		this.list_ = new ArrayList<Vertex>(size);
	}
	
	// empty list constructor
	public RVertexList() {
		this(0);
	}

	// singleton constructor
	public RVertexList(Vertex v) {
		this(1);
		this.list_.add(v);
	}

	// full list constructor
	public RVertexList(Collection<Vertex> c) {
		this(c.size());
		this.list_.addAll(c);
	}

	// clone : copy the internal list of vertices
	//         but not the vertices themselves
	//
	public RegistryValue klone() {
		RVertexList clone = new RVertexList(list_.size());
		for (Vertex v : list_) {
			clone.addVertex(v);
		}
		return clone;
	}

	public String serialize() {
		throw new IllegalValueException();
	}

	public boolean toBoolean() throws IllegalValueException {
		throw new IllegalValueException();
	}

	public float toFloat() throws IllegalValueException {
		throw new IllegalValueException();
	}

	public int toInt() throws IllegalValueException {
		throw new IllegalValueException();
	}
	
	// equals and hashcode
	
	public boolean equals(Object o) {
		return (  (o instanceof RVertexList)
			    ? this.list_.equals(((RVertexList) o).list_)
				: false);
	}

	public int hashCode() {
		return this.list_.hashCode();
	}
	
	// Iterable API

	public Iterator<Vertex> iterator() {
		return this.list_.iterator();
	}
	
	// added functions

	public void addVertex(Vertex v) {
		list_.add(v);
	}

	public Vertex getVertex(int index) throws IndexOutOfBoundsException {
		return list_.get(index);
	}
	
	public Vertex lastVertex() throws IndexOutOfBoundsException {
		return list_.get(list_.size()-1);
	}

	public Vertex setVertex(int index, Vertex v) throws IndexOutOfBoundsException {
		return list_.set(index, v);
	}
	
	public Vertex removeVertex(int index) throws IndexOutOfBoundsException {
		return list_.remove(index);
	}
	
	public void clear() {
		list_.clear();
	}

	public int size() {
		return list_.size();
	}

	/**
	 * a human readable version of toString()
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		for (Vertex v : list_) {
			buffer.append(v.toString() + ", ");
		}
		int n = (buffer.length() > 2 ? buffer.length() - 2 : 0);
		return "(" + buffer.substring(0, n).trim() + ")";
	}
	
	/**
	 * Lookup the RVertexList associated to key in registry and
	 * create an empty RVertexList if it does not exist.
	 * This helper function is equivalent to :
	 * <code>
	 * if (r.has(key))<br>
	 *    return (RVertexList) r.get(key);<br>
	 * else<br>
	 *    return (RVertexList) r.set(key, new RVertexList());<br>
	 * </code>
	 */
	public static final RVertexList getVertexList(Registry r, String key) {
		RVertexList tuple = (RVertexList) r.get(key);
		if (tuple == null) { 
			tuple = new RVertexList();
			r.set(key, tuple);
		}
		return tuple;
	}
	
	
}
