package helix.graph.model.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import helix.graph.model.IllegalValueException;
import helix.graph.model.Registry;
import helix.graph.model.RegistryValue;
import helix.graph.model.Vertex;

/**
 * a memory implementation of RegistryValue for set of Vertices
 * @author viari
 *
 */
public class RVertexSet implements RegistryValue, Iterable<Vertex> {

	protected HashSet<Vertex> set_;
		
	// API

	// empty set constructor
	public RVertexSet() {
		this.set_ = new HashSet<Vertex>(1);
	}

	// singleton constructor
	public RVertexSet(Vertex v) {
		this.set_ = new HashSet<Vertex>(1);
		this.set_.add(v);
	}

	// full list set constructor
	public RVertexSet(Collection<Vertex> c) {
		this.set_ = new HashSet<Vertex>(c.size());
		this.set_.addAll(c);
	}

	// clone : copy the internal set of vertices
	//         but not the vertices themselves
	//
	public RegistryValue klone() {
		RVertexSet clone = new RVertexSet();
		for (Vertex v : this) {
			clone.addVertex(v);
		}
		return clone;
	}

	public String serialize() {
		throw new IllegalValueException();
	}

	public boolean toBoolean() throws IllegalValueException {
		throw new IllegalValueException();
	}

	public float toFloat() throws IllegalValueException {
		throw new IllegalValueException();
	}

	public int toInt() throws IllegalValueException {
		throw new IllegalValueException();
	}
	
	// equals and hashcode
	
	public boolean equals(Object o) {
		return (  (o instanceof RVertexSet)
			    ? this.set_.equals(((RVertexSet) o).set_)
				: false);
	}

	public int hashCode() {
		return this.set_.hashCode();
	}

	// Iterable API

	public Iterator<Vertex> iterator() {
		return this.set_.iterator();
	}

	// added functions

	public boolean hasVertex(Vertex v) {
		return set_.contains(v);
	}

	public void addVertex(Vertex v) {
		set_.add(v);
	}

	public void removeVertex(Vertex v) {
		set_.remove(v);
	}

	public void clear() {
		set_.clear();
	}
	
	public int size() {
		return set_.size();
	}

	/**
	 * a human readable version of toString()
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		for (Vertex v : set_) {
			buffer.append(v.toString() + ", ");
		}
		int n = (buffer.length() > 2 ? buffer.length() - 2 : 0);
		return "{" + buffer.substring(0, n).trim() + "}";
	}
	
	/**
	 * Lookup the RVertexSet associated to key in registry and
	 * create an empty RVertexSet if it does not exist.
	 * This helper function is equivalent to :
	 * <code>
	 * if (r.has(key))<br>
	 *    return (RVertexSet) r.get(key);<br>
	 * else<br>
	 *    return (RVertexSet) r.set(key, new RVertexSet());<br>
	 * </code>
	 */
	public static final RVertexSet getVertexSet(Registry r, String key) {
		RVertexSet set = (RVertexSet) r.get(key);
		if (set == null) { 
			set = new RVertexSet();
			r.set(key, set);
		}
		return set;
	}
}
