package helix.graph.model.impl.memory;

import helix.graph.model.Vertex;
import helix.graph.model.Edge;
import helix.graph.model.NoSuchVertexException;
import helix.graph.model.SameVertexException;

public class MemEdge extends MemRegistry implements Edge {

	protected Vertex v1_, v2_;

    /* 
     * the constructor is protected
     * use Graph.addEdge(v1, v2) instead
     */
	protected MemEdge(Vertex v1, Vertex v2) {
		if (v1.equals(v2))
			throw new SameVertexException(v1.toString());
		v1_ = v1;
		v2_ = v2;
	}

	/*
	 * API
	 * 
	 */

	public Vertex v1() {
		return v1_;
	}
	
	public Vertex v2() {
		return v2_;
	}

	public boolean hasVertex(Vertex v) {
		return v.equals(v1_) || v.equals(v2_);
	}

	public Vertex getVertexTo(Vertex v) {
		if (v.equals(v1_))
			return v2_;
		if (v.equals(v2_))
			return v1_;
		throw new NoSuchVertexException(v.toString());
	}

	public Vertex[] toArray() {
		Vertex[] v = new Vertex[2];
		v[0] = v1_;
		v[1] = v2_;
		return v;
	}

	public int hashCode() {
		return v1_.hashCode() + v2_.hashCode();
	}

	public boolean equals(Edge e) {
		return e.hasVertex(v1_) && e.hasVertex(v2_);
	}

	/**
	 * a human readable version of toString()
	 */
	public String toString() {
		return "Edge[" + v1_.toString() + "," + v2_.toString() + "]";
	}
	
}
