package helix.graph.model.impl.memory;

import helix.graph.model.DuplicateValueException;
import helix.graph.model.GSet;
import helix.graph.model.GSetFactory;
import helix.graph.model.GraphElement;
import helix.graph.model.RegistryValue;
import helix.graph.model.impl.RString;

import java.util.HashSet;
import java.util.Hashtable;

public class MemGSet<E extends GraphElement> extends HashSet<E> implements GSet<E> {

	private static final long serialVersionUID = 1L;
	
	protected Hashtable<RegistryValue, E> uniqueTable_ = null;
	protected Hashtable<RegistryValue, GSet<E>> multipleTable_ = null;
	protected String uniqueKey_ = null;
	protected String multipleKey_ = null;
	
	/*
	 * internal hashing utilities
	 */
	
	protected void hashUnique_(String key) throws DuplicateValueException {
		uniqueKey_ = null;
		uniqueTable_ = new Hashtable<RegistryValue, E>();
		for (E elt : this) {
			RegistryValue value = elt.get(key);
			if (value == null)
				continue; // skip over 
			if (uniqueTable_.containsKey(value))
				throw new DuplicateValueException("duplicate value for key: " + key);
			uniqueTable_.put(value, elt);
		}
		uniqueKey_ = key;
	}
	
	protected void hashMultiple_(String key) {
		multipleKey_ = null;
		multipleTable_ = new Hashtable<RegistryValue, GSet<E>>();
		for (E elt : this) {
			RegistryValue value = elt.get(key);
			if (value == null)
				continue; // skip over 
			GSet<E> set = (  multipleTable_.containsKey(value) 
			               ? multipleTable_.get(value) 
			               : this.factory().make());
			set.add(elt);
			multipleTable_.put(value, set);
		}
		multipleKey_ = key;
	}
	
	private static final void checkKeyValue_(Object key, Object value) 
	  throws IllegalArgumentException {
		if (key == null)
			throw new IllegalArgumentException("null registry key");
		if (value == null)
			throw new IllegalArgumentException("null registry value");
	}
	
	private final void resetTables_() {
		uniqueKey_ = null;
		multipleKey_ = null;
		uniqueTable_ = null;
		multipleTable_ = null;
	}

	/*
	 * API
	 */

	public GSetFactory<E> factory() {
		return new MemGSetFactory<E>();
	}

	public void addIterable(Iterable<E> g) {
		for(E e : g) {
			this.add(e);
		}
	}

	public E findUnique(String registryKey, RegistryValue registryValue)
	  throws DuplicateValueException {

		if (RehashKey.equals(registryKey)) { // force rehash next time
			resetTables_();
			return null;
		}

		checkKeyValue_(registryKey, registryValue);
		
		if (! registryKey.equals(uniqueKey_)) {
			hashUnique_(registryKey);
		}
		return uniqueTable_.get(registryValue);
	}

	public E findUnique(String registryKey, Object anyValue)
	  throws DuplicateValueException {

		if (anyValue == null)
			throw new IllegalArgumentException("null registry value");
		
		return this.findUnique(registryKey, new RString(anyValue.toString()));
	}

	public GSet<E> findMultiple(String registryKey, RegistryValue registryValue) {

		if (RehashKey.equals(registryKey)) { // force rehash next time
			resetTables_();
			return new MemGSet<E>() ;
		}

		checkKeyValue_(registryKey, registryValue);

		if (! registryKey.equals(multipleKey_)) {
			hashMultiple_(registryKey);
		}
		
		GSet<E> result = multipleTable_.get(registryValue);
		return (result == null ? new MemGSet<E>() : result);
	}

	public GSet<E> findMultiple(String registryKey, Object anyValue) {

		if (anyValue == null)
			throw new IllegalArgumentException("null registry value");
		
		return this.findMultiple(registryKey, new RString(anyValue.toString()));
	}
}
