package helix.graph.model.impl.memory;


import helix.graph.model.GSetFactory;
import helix.graph.model.GraphFactory;
import helix.graph.model.Registry;
import helix.graph.model.SameVertexException;
import helix.graph.model.Vertex;
import helix.graph.model.Edge;
import helix.graph.model.NoSuchVertexException;
import helix.graph.model.DuplicateEdgeException;
import helix.graph.model.impl.AbstractGraph;

public class MemGraph extends AbstractGraph {

	protected Registry registry_ = new MemRegistry();
	
	/*
	 * MemGraph constructors API
	 */
	
	public MemGraph() {
	}
	
	public GraphFactory factory() {
		return MemGraphFactory.instance();
	}
	
	public GSetFactory<Vertex> vertexSetFactory() {
		return new MemGSetFactory<Vertex>();
	}

	public GSetFactory<Edge> edgeSetFactory() {
		return new MemGSetFactory<Edge>();
	}
	/*
	 * Registry API
	 */

	public Registry registry() {
		return registry_;
	}
	
	/*
	 * Graph API
	 */

	protected MemVertex checkVertex(Vertex v)  throws NoSuchVertexException {
		if (! vertices_.contains(v))
			throw new NoSuchVertexException(v.toString());
		return (MemVertex) v;
	}

	public Vertex addVertex() {
		MemVertex v = new MemVertex();
		vertices_.add(v);
		return v;
	}
	
	public Edge addEdge(Vertex v1, Vertex v2)
	  throws NoSuchVertexException, SameVertexException, DuplicateEdgeException {
		MemVertex mv1 = checkVertex(v1);
		MemVertex mv2 = checkVertex(v2);
		
		MemEdge e = new MemEdge(v1, v2);

		mv1.addEdge(e);
		mv2.addEdge(e);

		return e;
	}

	public void removeEdge(Vertex v1, Vertex v2) throws NoSuchVertexException {
		MemVertex mv1 = checkVertex(v1);
		MemVertex mv2 = checkVertex(v2);

		Edge e = mv1.getEdgeTo(mv2);
		if (e != null) {
		  mv1.removeEdge(e);
		  mv2.removeEdge(e);
		}
	}

}
