package helix.graph.model.impl.memory;

import java.util.Hashtable;

import helix.graph.model.DuplicateEdgeException;
import helix.graph.model.Edge;
import helix.graph.model.EdgeToVertexIterable;
import helix.graph.model.Registry;
import helix.graph.model.Vertex;
import helix.graph.util.EmptyIterable;

public class MemVertex extends MemRegistry implements Vertex {

	private static EmptyIterable<Edge> EmptyIterable_ = new EmptyIterable<Edge>();
	
	// edges_ : Hashtable of edges incident to this vertex
	// is not initialized to save space
	protected Hashtable<Vertex, Edge> edges_ = null;

	/**
	 * the constructor is protected
	 * use Graph.addVertex() instead
	 */
	protected MemVertex() {
	}

	protected final void assertEdges() {
		if (edges_ == null)
			edges_ = new Hashtable<Vertex, Edge>(0);
	}
	
	protected void addEdge(Edge e) throws DuplicateEdgeException {
		assertEdges();
		Vertex key = e.getVertexTo(this); // key is the associated vertex

		if (edges_.containsKey(key))
		  throw new DuplicateEdgeException(key.toString());
		
		edges_.put(key, e);
	}

	protected void removeEdge(Edge e) {
		if (edges_ == null)
			return;
		Vertex key = e.getVertexTo(this); // key is the associated vertex
		edges_.remove(key);
	}
	
	/*
	 * Vertex API
	 */

	public int degree() {
		return (edges_ == null ? 0 : edges_.size());
	}
	
	public Edge getEdgeTo(Vertex v) {
		return (edges_ == null ? null : edges_.get(v));
	}

	public boolean hasEdgeTo(Vertex v) {
		return ((edges_ != null) &&  edges_.containsKey(v));
	}

	public Iterable<Edge> edges() {
		return (edges_ == null ? EmptyIterable_ : edges_.values());
	}

	public Iterable<Vertex> neighbors() {
		return new EdgeToVertexIterable(edges(), this);
	}

	/**
	 * a human readable version of toString()
	 */
	public String toString() {
		// try with labels
		if (registry().has(Registry.Label)) {
			return registry().get(Registry.Label).toString();
		}
		// else do it the old way
		return super.toString();
	}
}
