package helix.graph.model.impl.memory;

import java.util.Iterator;
import java.util.Set;

import helix.graph.model.DuplicateValueException;
import helix.graph.model.GSet;
import helix.graph.model.Vertex;

import org.junit.*;

import static org.junit.Assert.*;

public class TestMemGSet {


	@Test
	public void testGSet() {
		GSet<Vertex> s1 = new MemGSet<Vertex>();
		Vertex v = new MemVertex();
		s1.add(v);
		assertEquals("MemGSet<Vertex> size", 1, s1.size());
	}

	@Test
	public void testFactory() {
		GSet<Vertex> s1 = new MemGSet<Vertex>();
		GSet<Vertex> s2 = s1.factory().make();
		s2.add(new MemVertex());
		assertEquals("MemGSet<Vertex> size", 1, s2.size());
	}

	@Test
	public void testSuperFactory() {
		GSet<Vertex> s1 = new MemGSet<Vertex>();
		Set<GSet<Vertex>> s2 = s1.factory().makeSuper();
		s2.add(s1);
		s1.add(new MemVertex());
		s1.add(new MemVertex());
		assertEquals("GSet<Vertex> size", 2, s1.size());
		assertEquals("GSet<GSet<Vertex>> size", 1, s2.size());
		Iterator<GSet<Vertex>> it = s2.iterator();
		GSet<Vertex> s3 = it.next();
		assertEquals("same set", s1, s3);
	}

	@Test
	public void testFindSimple() {
		GSet<Vertex> s1 = new MemGSet<Vertex>();
		Vertex v1 = new MemVertex();
		Vertex v2 = new MemVertex();
		Vertex v3 = new MemVertex();
		s1.add(v1);
		s1.add(v2);
		s1.add(v3);
		v1.set("color", "red");
		v2.set("color", "red");
		v3.set("color", "blue");
		v1.set("index", 1);
		v2.set("index", 2);
		v3.set("index", 3);
		Vertex u1 = s1.findUnique("index", 1);
		assertEquals("v1", v1, u1);
		Vertex u2 = s1.findUnique("index", 2);
		assertEquals("v2", v2, u2);
		Vertex u3 = s1.findUnique("index", 3);
		assertEquals("v3", v3, u3);
		GSet<Vertex> found = s1.findMultiple("color", "red");
		assertEquals("red size", 2, found.size());
		boolean catched = false;
		try {
			s1.findUnique("color", "red");
		}
		catch (DuplicateValueException dve) {
			catched = true;
		}
		assertTrue("DuplicateValueException catched", catched);
	}

	@Test
	public void testFindSpecial() {
		GSet<Vertex> s1 = new MemGSet<Vertex>();
		Vertex v1 = new MemVertex();
		s1.add(v1);
		v1.set("index", 1);
		v1.set("color", "1");
		Vertex u1;
		u1 = s1.findUnique("index", 1);
		assertEquals("v1", v1, u1);
		u1 = s1.findUnique("index", "1");
		assertEquals("v1", v1, u1);
		u1 = s1.findUnique("color", "1");
		assertEquals("v1", v1, u1);
		u1 = s1.findUnique("color", 1);
		assertEquals("v1", v1, u1);
		
		u1 = s1.findUnique("noindex", 1);
		assertEquals("v1", null, u1);
		u1 = s1.findUnique("index", 2);
		assertEquals("v1", null, u1);
		u1 = s1.findUnique("index", true);
		assertEquals("v1", null, u1);
		u1 = s1.findUnique("index", false);
		assertEquals("v1", null, u1);
	}

	@Test(expected= IllegalArgumentException.class)
	public void testThrow1() {
		GSet<Vertex> s1 = new MemGSet<Vertex>();
		s1.findUnique(null, "toto");
	}

	@Test(expected= IllegalArgumentException.class)
	public void testThrow2() {
		GSet<Vertex> s1 = new MemGSet<Vertex>();
		s1.findUnique("index", null);
	}

	@Test(expected= IllegalArgumentException.class)
	public void testThrow3() {
		GSet<Vertex> s1 = new MemGSet<Vertex>();
		s1.findMultiple("index", null);
	}

	

}
