package helix.graph.util;



import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;

import thirdparty.ostermiller.util.CmdLn;
import thirdparty.ostermiller.util.CmdLnOption;
import thirdparty.ostermiller.util.CmdLnResult;


public class CommandLine {

	protected String usage_ = "usage:";
	
	protected CmdLn command_;
	
	protected Hashtable<String, CmdLnOption> table_ = new Hashtable<String, CmdLnOption>();

	protected HashSet<String> userSet_ = new HashSet<String>();
	
	private static final List<String> emptyList_ = new ArrayList<String>(0);

	/**
	 * constructor
	 */
	public CommandLine(String args[]) {
		command_ = new CmdLn(args);
		
		// add default options
		addOption("help", 'h').setNoArguments().setDescription("print this help and exit");
	}

	/**
	 * set usage description
	 */
	public void setUsage(String usage) {
		usage_ = usage;
		command_.setDescription(usage);
	}

	/**
	 * get usage description
	 */
	public String getUsage() {
		return usage_;
	}
	
	/**
	 * get internal com.Ostermiller.util.CmdLn
	 */
	public CmdLn getCommand() {
		return command_;
	}

	/**
	 * add option
	 */
	public CmdLnOption addOption(String longName, Character shortName) {

		CmdLnOption option = new CmdLnOption(longName, shortName);

		option.setUserObject(longName);

		command_.addOption(option);		

		table_.put(longName, option);
		
		return option;
	}

	/**
	 * get registered option
	 */
	public CmdLnOption getOption(String longName) {
		return table_.get(longName);
	}

	/**
	 * parse command line
	 */
	public void parse() throws CommandLineException {

		List<CmdLnResult> results;

		// parse command line
		
		try {
			results = command_.getResults();
		} catch (Exception e) {
			throw new CommandLineException(e.getMessage());
		}

		// invoke stubs
		
		for (CmdLnResult result : results) {

			List<String> args = result.getArguments();
			String name = (String) result.getOption().getUserObject();

			try {
				Method m = this.getClass().getMethod("set_" + name, List.class);
				m.invoke(this, args == null ? emptyList_ : args);
				userSet_.add(name);
			} catch (Exception e) {
				// e.printStackTrace();
				throw new CommandLineException("incorrect option value for " + name + " : " + e.getCause().getMessage());
			}

		}
	}
	
	/**
	 * get remaining arguments after all options have been removed
	 */
	
	public List<String> getNonOptionArguments() {
    	return command_.getNonOptionArguments();
	}
	
	/**
	 * tell if option has been set by user
	 */
	public boolean isSetByUser(String name) {
		return userSet_.contains(name);
	}

	/*
	 * some default stubs
	 */

	/**
	 * default stub for help invocation
	 */

	public void set_help(List<String> args) {
		command_.printHelp();
		System.exit(0);
	}

	/*
	 * some helper functions : to parse arguments values
	 */

	/**
	 * helper function  to parse arguments values
	 * boolean -> String "on" "off"
	 */
	public static String boolToString(boolean b) {
		return (b ? "on" : "off");
	}
	
	/**
	 * helper function  to parse arguments values
	 * String "on" "off" "true" "false" "1" "0" -> boolean
	 */
	private static final String[] FALSE_STRINGS = {"off", "false", "0"};
	
	public static boolean stringToBool(String s) {
		for (String key : FALSE_STRINGS) {
			if (key.compareToIgnoreCase(s) == 0)
				return false;
		}
		return true; // everything else is true
	}
	
	/**
	 * helper function to catch infinite integer value
	 * String "inf(inity)?" -> infiniteValue
	 */
	private static final String INFINITE_PATTERN = "inf(inity)?";
	
	public static int stringToInt(String s, int infiniteValue) {
		if (s.toLowerCase().matches(INFINITE_PATTERN))
			return infiniteValue;
		else
			return Integer.parseInt(s);
	}
	
}
