package helix.graph.util;

import java.util.logging.ConsoleHandler;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

public class LogUtil {

	/**
	 * the default Logger name
	 */
	protected static final String DefaultLoggerName = "LXGraphLogger";
	
	/*
	 * internal utility : initialize logger behavior
	 */
	protected static final Logger initLogger_(Logger logger) {
		
		if (logger.getHandlers().length != 0) // already fixed
			return logger;
			
		// no parent handler
		logger.setUseParentHandlers(false);

		// setup default handler
		Handler dftHandler = new ConsoleHandler();
		dftHandler.setFormatter(new SimpleLogFormatter());
		logger.addHandler(dftHandler);
		
		// default level
		logger.setLevel(Level.INFO);
		
		return logger;
	}
	
	/**
	 * protected constructor : use the getLogger factories instead
	 */
	protected LogUtil() {
	}

	/*
	 * getLogger methods
	 */

	public static Logger getLogger(String name) {
		return initLogger_(Logger.getLogger(name));
	}
	
	public static Logger getLogger(String name, String resourceBundleName) {
		return initLogger_(Logger.getLogger(name, resourceBundleName));
	}

	public static Logger getLogger() {
		return getLogger(DefaultLoggerName);
	}
	
	public static Logger getUniqueLogger(String prefix) {
		String name;
		int i = 0;
		do {
			name = prefix + "_" + i++;
		} while (LogManager.getLogManager().getLogger(name) != null);
		return getLogger(name);
	}

	public static Logger getUniqueLogger() {
		return getUniqueLogger(DefaultLoggerName);
	}
	
	/**
	 * enable/disable Logging
	 * return previous state
	 */
	public static boolean setVerbose(Logger logger, boolean enable) {
		boolean previous = isVerbose(logger);
		logger.setLevel(enable ? Level.INFO : Level.SEVERE); // could be Level.OFF
		return previous;
	}

	/**
	 * get Logging verbosity
	 */
	public static boolean isVerbose(Logger logger) {
		return logger.isLoggable(Level.INFO);
	}

	/**
	 * set log formatter header
	 * return previous header
	 */
	public static String setHeader(Logger logger, String header) {
		String previous = "";
		for (Handler h : logger.getHandlers()) {
			Formatter fmt = h.getFormatter();
			if (fmt instanceof SimpleLogFormatter) {
				SimpleLogFormatter sfmt = (SimpleLogFormatter) fmt;
				previous = sfmt.getHeader();
				sfmt.setHeader(header);
			}
		}
		return previous;
	}
}
