package helix.graph.util;

import java.text.DecimalFormat;
import java.util.logging.Level;
import java.util.logging.Logger;

public class MemoryBench {

	// defaut output format
	protected static DecimalFormat format_ = new DecimalFormat("###,###");

	protected static int gcloop_ = 10;
	
	protected static final void garbageCollect_(int loop) {
		for (int i = 0 ; i < loop ; i++)
			System.gc();
	}

	protected static final Runtime runtime_(boolean accurate) {
		Runtime r = Runtime.getRuntime();
		garbageCollect_(accurate ? gcloop_ : 0);
		return r;
	}
	
	public static void setAccuracy(int loop) {
		gcloop_ = loop;
	}

	public static long totalMemory(boolean accurate) {
		return runtime_(accurate).totalMemory();
	}
	
	public static long totalMemory() {
		return totalMemory(true);
	}
	
	public static long freeMemory(boolean accurate) {
		return runtime_(accurate).freeMemory();
	}

	public static long freeMemory() {
		return freeMemory(true);
	}

	public static long usedMemory(boolean accurate) {
		Runtime r = runtime_(accurate);
		return r.totalMemory() - r.freeMemory();
	}

	public static long usedMemory() {
		return usedMemory(true);
	}
	
	/**
	 * log total and used memory on Logger
	 * using accurate mode
	 */
	public static void log(Logger logger, Level level, String header) {
		String total = format_.format(totalMemory(true));
		String used = format_.format(usedMemory(true));
		logger.log(level, header + " total memory : " + total
				                 + " used memory : " + used);
	}

	/**
	 * log current value on Logger at INFO level
	 */
	public static void log(Logger log, String header) {
		log(log, Level.INFO, header);
	}

}
