package helix.graph.util;

import java.text.DecimalFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;


public class MemoryWatcher extends Thread {

	// default timing interval in milliseconds
	protected static final long DFT_INTERVAL = 5000;
	
	// defaut output format
	protected static DecimalFormat format_ = new DecimalFormat("###,###");
	
	protected long interval_ = DFT_INTERVAL;
	protected boolean accurate_ = false;
	protected boolean run_ = true;
	protected long maxused_ = 0;
	protected Logger logger_ = null;

	public MemoryWatcher(Logger logger) {
		logger_ = logger;
	}

	public MemoryWatcher(Logger logger, long millisec) {
		this(logger);
		interval_ = millisec;
	}

	public MemoryWatcher(Logger logger, long millisec, boolean accurate) {
		this(logger, millisec);
		accurate_ = accurate;
	}
	
	public void run() {
		Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
		while (run_) {
			try {
				long used = MemoryBench.usedMemory(accurate_);
				if (used > maxused_) {
					maxused_ = used;
				}
				this.log(new Date() + " :");
				Thread.sleep(interval_);
			} 
			catch (InterruptedException e) {
			}
		}
	}
	
	public void stopRun() {
		run_ = false;
	}
	
	public long maxUsedMemory() {
		return maxused_;
	}

	public void log(String header) {
		String used = (maxused_ > 0) ? format_.format(maxused_) : "undefined";
		logger_.log(Level.INFO, "Memory dog " + header + " used memory : " + used +
				" (" + (accurate_ ? "" : "non ") + "accurate)");
	}
	
}
