package helix.graph.util;

import helix.graph.io.DimacsExtendedReader;
import helix.graph.io.GraphReader;
import helix.graph.model.Graph;
import helix.graph.model.GraphFactory;
import helix.graph.model.Registry;
import helix.graph.model.Vertex;
import helix.graph.model.impl.RVertexList;
import helix.graph.model.impl.memory.MemGraphFactory;

import java.io.IOException;
import java.io.StringReader;

import org.junit.Test;


public class TestGraphDebugger {

	@Test
	public void testGraphDebug() throws IOException {

		String sin = "p test\nn label=a1 colors=10\nn label=a2 colors=10\nn label=a3 colors=10\nn label=a4 colors=10\nn label=a5 colors=10\n" +
		"n label=b1 colors=100\nn label=b2 colors=100\nn label=b3 colors=100\nn label=b4 colors=100\n" +
		"e 1 2 colors=10\ne 2 3 colors=10\ne 3 4 colors=10\ne 4 5 colors=10\n" +
		"e 6 7 colors=100\ne 7 8 colors=100\ne 8 9 colors=100\n" +
		"e 1 6 colors=1\ne 2 7 colors=1\ne 3 7 colors=1\ne 5 7 colors=1\ne 4 9 colors=1\n";

		StringReader sreader = new StringReader(sin);
		GraphFactory factory = MemGraphFactory.instance();
		GraphReader reader = new DimacsExtendedReader(sreader, factory);
		Graph graph = reader.read();

		GraphDebugger gd = new GraphDebugger();
		
		gd.log(graph);
		
		Vertex a1 = graph.findVertex(Registry.Label, "a1");
		Vertex a2 = graph.findVertex(Registry.Label, "a2");
		Vertex b1 = graph.findVertex(Registry.Label, "b1");
		
		RVertexList tuple = new RVertexList();
		tuple.addVertex(a1);
		tuple.addVertex(a2);
		tuple.addVertex(b1);
		a1.set("Nodes", tuple);
		
		tuple = new RVertexList();
		a2.set("Nodes", tuple);

		gd.addVertexAttribute(Registry.Label);
		gd.addVertexAttribute(Registry.Colors);
		gd.addVertexAttribute("Nodes");
		
		gd.log(graph, "With colors and Nodes");
		
		gd.removeVertexAttribute(Registry.Colors);
		gd.removeVertexAttribute("Nodes");
		
		gd.log(graph, true, "With edges");
		
	}

}
