package helix.graph.util;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;


public class TestIPermutation {

	private static final int fact_(int n) {
		int f = 1;
		for (int i = 1 ; i <= n ; i++) {
			f *= i;
		}
		return f;
	}
	
	private static final void printList_(List<String> lst) {
		for (String s : lst) {
			System.out.print(s + " ");
		}
		System.out.println();
	}

	@Test
	public void testIPermutation() {
		IPermutation p1 = new IPermutation(10);
		IPermutation p2 = new IPermutation(0, 9);
		IPermutation p3 = new IPermutation(1, 10);
		assertTrue("p1 identity", p1.isIdentity());
		assertTrue("p2 identity", p2.isIdentity());
		assertEquals("p1 == p2", p1, p2);
		assertFalse("p1 != p3", p1.equals(p3));
		assertEquals("p1 == p3.shift()", p1, p3.shift());
	}

	@Test
	public void testIPermutationParser() {
		IPermutation p1 = new IPermutation("1 2 3 4");
		assertEquals("p1 size", 4, p1.size());
		assertTrue("p1.shift() identity", p1.shift().isIdentity());
	}

	@Test
	public void testIPermutationApply() {
		ArrayList<String> lst = new ArrayList<String>();
		lst.add("a"); lst.add("b"); lst.add("c");
		ArrayList<String> chk = new ArrayList<String>();
		chk.add("c"); chk.add("a"); chk.add("b");
		IPermutation p = new IPermutation("2 0 1");
		List<String> res = p.apply(lst);
		assertEquals("permuted list", chk, res);
	}
	
	@Test
	public void testIPermutationIterator() {
		ArrayList<String> lst = new ArrayList<String>();
		lst.add("a");
		lst.add("b");
		lst.add("c");
		IPermutationIterator iter = new IPermutationIterator(lst.size());
		int n = 0;
		while (iter.hasNext()) {
			List<String> res = iter.next().apply(lst);
			printList_(res);
			n++;
		}
		assertEquals("perm size", fact_(lst.size()), n);
	}

	/*
	// TODO test for combination
	@Test
	public void testICombinationIterator() {
		ArrayList<String> lst = new ArrayList<String>();
		lst.add("a");
		lst.add("b");
		lst.add("c");
		lst.add("d");
		lst.add("e");
		ICombinationIterator iter = new ICombinationIterator(lst.size(), 3);
		int n = 0;
		while (iter.hasNext()) {
			List<String> res = iter.next().apply(lst);
			printList_(res);
			n++;
		}
		// assertEquals("perm size", fact_(lst.size()), n);
	}
	*/
}
